#ifndef CONFIG_H
#define CONFIG_H
#include <string>
#include <map>
#include <fileio.h>
#include "config_classes.h"
#ifdef Revision
#undef Revision
#endif
#define Revision "config.h $Revision: 1.2 $"
/**
 * Хранение конфигурации. Допустимо использование нескольких файлов конфигурациию.
 * Для загрузки конфигурации используется интерфейс драйверов. Допустимо создание и использование одновременно нескольких хранилищ конфигурации с разными (или одинаковыми) драйверами.
 * Обеспечивается многопоточная безопасность.
 *@author Denis A. Birukov aka Messenger of death
 *@date 18.08.2005
 */
/**
 * Фнукция производит создание хранилища конфигурации с указанным именем, и для этого используется указанный драйвер.
 * @param name Имя создаваемого хранилища
 * @param driver Имя драйвера
 * @param replace если установлен в true, то в случае, если хранилище с заданным именем уже существует, оно будет удалено.
 * @return true в случае успеха. false в случае неудачи. Причина неудачи может быть получена через _get_last_error();
 * @author Denis A. Birukov aka Messenger of death
 * @date 18.08.2005
 */
extern "C" bool config_create ( std::string name, std::string driver, bool replace = false );

/**
 * Функция удаляет хранилище конфигурации. При этом производится деинициализация драйвера.
 * @param name Имя удаляемого хранилища
 * @return true в случае успеха. false в случае неудачи. Причина неудачи может быть получена через _get_last_error();
 * @author Denis A. Birukov aka Messenger of death
 * @date 18.08.2005
 */
extern "C" bool config_remove ( std::string name );

/**
 * Возвращает указатель на интерфейс конфигуратора
 * @param name Имя хранилища, интерфейс к которому необходимо получить
 * @return Указатель на интерфейс хранилища, 0 в случае ошибки. 
 * @author Denis A. Birukov aka Messenger of death
 * @date 18.08.2005
 */
extern "C" Config_Interface* config_get_interface ( std::string name );

/**
  * Возвращает значение из конфигурации
  * @param conf Имя хранилища из которого требуется получить значение
  * @param name Имя значения.
  * @param def Значение по умолчанию.
  * @return Запрашиваемое значение конфига
  * @author Denis A. Birukov aka Messenger of death
  * @date 23.08.2005
 */
extern "C" std::string config_get_value ( std::string conf, std::string name, std::string def );

/**
   * Устанавливает значение в конфигурации 
   * @param conf Имя хранилища из которого требуется получить значение
   * @param name Имя значения.
   * @param val Значение .
   * @param replace флаг перезаписи значения, если оно уже существует. По умолчанию значение перетирается.
   * @return true в случае успех, false иначе.
   * @author Denis A. Birukov aka Messenger of death
   * @date 23.08.2005
 */
extern "C" bool config_set_value ( std::string conf, std::string name, std::string val, bool replace = true );

/**
 * Функция возвращает ссылку на текст предыдущей ошибки.
 * @return текст ошибки. Если ошибки небыло - строка будет пустой.
 * @author Messenger of death
 * @date 18.08.2005
 */
extern "C" std::string config_error();

/**
* Получение версии библиотеки
* @return строку, содержащую версию библиотеки
* @author Messenger of death
* @date 30.08.2005
*/
extern "C" std::string config_version();


#endif
