#include <iostream>
#include <fileio.h>
#include <stdio.h>
#include "config.h"
#include "config_classes.h"

const char *header = "#pragma once\n"
                     "#include <string>\n"
                     "#include \"sceneobjectfabric.h\"\n";

const char *get_conf_int = "\t\t\tConfig_Interface *intf = config_get_interface ( storage_name );\n"
                           "\t\t\tif ( !intf )\n"
                           "\t\t\t{\n"
                           "\t\t\t\t_set_error ( \"Can't get config interface:\" + config_error() );\n"
                           "\t\t\t\treturn false;\n"
                           "\t\t\t}";
#define LINE(spacer, text) std::cout<<spacer<<text<<std::endl;
std::string accessors_list;

bool load_config ( char *path, char *name, FileIO::FileManager *fmanager )
{
	if ( !config_create ( name, "libtextconfig" ) )
	{
		std::cerr << "Error while trying to create config from libtextconfig: " << config_error() << std::endl;
		return false;
	}
	Config_Interface *intf = config_get_interface ( name );

	if ( !intf )
	{
		std::cerr << "Error with config";
		return false;
	}
	intf->_load ( path, fmanager );
	return true;
}

void write_includes ( Config_Interface *intf )
{
	char number[ 7 ];
	for ( int a = 0;a < 999999; ++a )
	{
		snprintf ( number, 7, "%d", a );
		std::string file = intf->_get_value ( std::string ( "object->includes->include" ) + number + "->file", "" );
		if ( file.empty() ) break;
		std::cout << "#include \"" << file << "\"" << std::endl;
	}
}
void write_functions ( const std::string &class_name, std::vector<std::string> &base_classes, Config_Interface *intf, const std::string ptype )
{
	std::cout << "//" << ptype << " functions" << std::endl;
	char number[ 7 ];
	for ( int a = 0;a < 999999; ++a )
	{
		snprintf ( number, 7, "%d", a );
		std::string name = intf->_get_value ( std::string ( "object->" ) + ptype + "->functions->function" + number + "->name", "" );
		std::string type = intf->_get_value ( std::string ( "object->" ) + ptype + "->functions->function" + number + "->type", "" );
		std::string define = intf->_get_value ( std::string ( "object->" ) + ptype + "->functions->function" + number + "->define", "no" );
		std::string parents_rule = intf->_get_value ( std::string ( "object->" ) + ptype + "->functions->function" + number + "->parents_ok", "none" );
		std::string is_expand = intf->_get_value ( std::string ( "object->" ) + ptype + "->functions->function" + number + "->is_expand", "true" );
		std::string virtual_state = intf->_get_value ( std::string ( "object->" ) + ptype + "->functions->function" + number + "->virtual", "no" );

		if ( name.empty() ) break;
		if ( type == "builtin" )
		{
			if ( name == "constructor" )
			{
				std::cout << "\t\t" << class_name << " ()";
				if ( define == "yes" )
				{
					int c = 0;
					for ( std::vector<std::string>::iterator it = base_classes.begin(); it != base_classes.end(); ++it )
					{
						std::cout << ( ( c == 0 ) ? " : " : ", " ) << *it << "()";
						++c;
					}
					//Проинициализируем слоты
					for ( int a = 0;a < 999999; ++a )
					{
						snprintf ( number, 7, "%d", a );
						std::string name = intf->_get_value ( std::string ( "object->" ) + "public" + "->slots->slot" + number + "->name", "" );
						std::string type = intf->_get_value ( std::string ( "object->" ) + "public" + "->slots->slot" + number + "->type", "" );
						if ( name.empty() ) break;
						if ( type == "virtual" ) continue;
						std::cout << ( ( c == 0 ) ? " : " : ", " ) << "INIT_SLOT(" << class_name << ", this, " << name << " )";
						++c;
					}
					for ( int a = 0;a < 999999; ++a )
					{
						snprintf ( number, 7, "%d", a );
						std::string name = intf->_get_value ( std::string ( "object->" ) + "protected" + "->slots->slot" + number + "->name", "" );
						std::string type = intf->_get_value ( std::string ( "object->" ) + "protected" + "->slots->slot" + number + "->type", "" );
						if ( name.empty() ) break;
						if ( type == "virtual" ) continue;
						std::cout << ( ( c == 0 ) ? " : " : ", " ) << "INIT_SLOT(" << class_name << ", this, " << name << " )";
						++c;
					}
					for ( int a = 0;a < 999999; ++a )
					{
						snprintf ( number, 7, "%d", a );
						std::string name = intf->_get_value ( std::string ( "object->" ) + "private" + "->slots->slot" + number + "->name", "" );
						std::string type = intf->_get_value ( std::string ( "object->" ) + "private" + "->slots->slot" + number + "->type", "" );
						if ( name.empty() ) break;
						if ( type == "virtual" ) continue;
						std::cout << ( ( c == 0 ) ? " : " : ", " ) << "INIT_SLOT(" << class_name << ", this, " << name << " )";
						++c;
					}

					std::cout << "{}" << std::endl;
				}
				else std::cout << ";" << std::endl;
			}
			else if ( name == "destructor" )
			{
				std::cout << "\t\t~" << class_name << "()";
				if ( define == "yes" ) std::cout << "{}" << std::endl;
				else std::cout << ";" << std::endl;
			}
			else if ( name == "draw" )
			{
				std::cout << "\t\tvirtual void draw()";
				if ( define == "yes" )
				{
					std::cout << std::endl << "\t\t{" << std::endl;
					for ( std::vector<std::string>::iterator it = base_classes.begin(); it != base_classes.end(); ++it )
					{
						std::cout << "\t\t\t" << *it << "::draw();" << std::endl;
					}
					if ( is_expand == "true" ) std::cout << "\t\t\treal_draw();" << std::endl;
					std::cout << "\t\t}" << std::endl;
					if ( is_expand == "true" ) std::cout << "\t\tvoid real_draw();" << std::endl;
				}
				else std::cout << ";" << std::endl;
			}
			else if ( name == "load" )
			{
				std::cout << "\t\t"<< ( virtual_state=="yes"?"virtual":"" ) <<" bool load ( Scene *scene, std::string storage_name, std::string storage_prefix, std::string scene_name, std::string object_name, FileIO::FileManager *fmanager)";
				if ( define == "yes" )
				{
					std::cout << std::endl << "\t\t{" << std::endl;
					std::cout << "\t\t\t_scene_name_ = scene_name;"<<std::endl;
					std::cout << "\t\t\t_object_name_ = object_name;"<<std::endl;
					std::cout << "\t\t\t_storage_name_ = storage_name;"<<std::endl;
					std::cout << "\t\t\t_storage_prefix_ = storage_prefix;"<<std::endl;
					std::cout << "\t\t\t_scene_ = scene;"<<std::endl;

					std::cout << get_conf_int << std::endl;
					std::cout << "\t\t\tbool state[" << base_classes.size() << "];" << std::endl;
					int c = 0;
					for ( std::vector<std::string>::iterator it = base_classes.begin(); it != base_classes.end(); ++it )
					{
						std::cout << "\t\t\tstate[" << c << "]=" << *it << "::load(scene, storage_name, storage_prefix, scene_name, object_name, fmanager);" << std::endl;
						++c;
					}
					if ( parents_rule == "all" )
					{
						std::cout << "\t\t\tfor(int a=0;a<" << base_classes.size() << "; ++a)" << std::endl;
						std::cout << "\t\t\t\tif(!state[a])return false;" << std::endl;
					}
					else if ( parents_rule == "one" )
					{
						std::cout << "\t\t\tbool s=false;" << std::endl;
						std::cout << "\t\t\tfor(int a=0;a<" << base_classes.size() << "; ++a)" << std::endl;
						std::cout << "\t\t\t\tif(state[a]){s=state[a]; break;}" << std::endl;
						std::cout << "\t\t\tif(!s)return false;" << std::endl;
					}

					//Загрузим переменные
					for ( int a = 0;a < 999999; ++a )
					{
						snprintf ( number, 7, "%d", a );
						std::string name = intf->_get_value ( std::string ( "object->input_params->param" ) + number + "->name", "" );
						std::string type = intf->_get_value ( std::string ( "object->input_params->param" ) + number + "->type", "" );
						std::string def = intf->_get_value ( std::string ( "object->input_params->param" ) + number + "->default", "" );
						if ( name.empty() ) break;
						if ( type == "text" )
						{
							std::cout << "\t\t\t" << name << " = " << "intf->_get_value(storage_prefix+\"" << class_name << "->" << name << "\", \"" << def << "\");" << std::endl;
						}
						else if ( type == "float" )
						{
							std::cout << "\t\t\t" << name << " = " << "atof(intf->_get_value(storage_prefix+\"" << class_name << "->" << name << "\", \"" << def << "\").c_str());" << std::endl;
						}
						else if ( type == "int" )
						{
							std::cout << "\t\t\t" << name << " = " << "strtol(intf->_get_value(storage_prefix+\"" << class_name << "->" << name << "\", \"" << def << "\").c_str(), (char **) NULL, 0);" << std::endl;
						}
						else if ( type == "unsigned int" )
						{
							std::cout << "\t\t\t" << name << " = " << "strtoul(intf->_get_value(storage_prefix+\"" << class_name << "->" << name << "\", \"" << def << "\").c_str(), (char **) NULL, 0);" << std::endl;
						}
						else if ( type == "bool" )
						{
							std::cout << "\t\t\t" << name << " = (intf->_get_value(storage_prefix+\"" << class_name << "->" << name << "\", \"" << def << "\")==\"true\")?true:false;" << std::endl;
						}
					}
					//Заполним таблицу слотов
					for ( int z = 0;z < 3;z++ )
					{
						std::string pptype;
						if ( z == 0 ) pptype = "public";
						else if ( z == 1 ) pptype = "private";
						else if ( z == 2 ) pptype = "protected";
						for ( int a = 0;a < 999999; ++a )
						{
							snprintf ( number, 7, "%d", a );
							std::string name = intf->_get_value ( std::string ( "object->" ) + pptype + "->slots->slot" + number + "->name", "" );
							std::string type = intf->_get_value ( std::string ( "object->" ) + pptype + "->slots->slot" + number + "->type", "" );
							if ( name.empty() ) break;
							if ( type == "virtual" ) continue;
							std::cout << "\t\t\t_slot_map_.push_back( std::make_pair(_scene_name_ + \"->\" + _object_name_ + \"->" << name << "\", &" << name << "));" << std::endl;
						}
					}
					//Init signals
					for ( int z = 0;z < 3;z++ )
					{
						std::string ptype;
						if ( z == 0 ) ptype = "public";
						else if ( z == 1 ) ptype = "private";
						else if ( z == 2 ) ptype = "protected";
						for ( int a = 0;a < 999999; ++a )
						{
							snprintf ( number, 7, "%d", a );
							std::string spname = intf->_get_value ( std::string ( "object->" ) + ptype + "->signals->signal" + number + "->name", "" );
							if ( spname.empty() ) break;
							std::cout << "\t\t\t"<<spname<<".set_name(_scene_name_ + \"->\" + _object_name_ + \"->"<<spname<<"\");" << std::endl;
						}
					}

					if ( is_expand == "true" ) std::cout << "\t\t\treturn real_load(scene, storage_name, storage_prefix, scene_name, object_name, fmanager);" << std::endl;
					else std::cout << "\t\t\treturn true;" << std::endl;
					std::cout << "\t\t}" << std::endl;
					if ( is_expand == "true" ) std::cout << "\t\tbool real_load ( Scene *scene, std::string storage_name, std::string storage_prefix, std::string scene_name, std::string object_name, FileIO::FileManager *fmanager);" << std::endl;
				}
				else std::cout << ";" << std::endl;
			}
		}
		else
		{
			std::string args = intf->_get_value ( std::string ( "object->" ) + ptype + "->functions->function" + number + "->args", "" );
			std::string ret = intf->_get_value ( std::string ( "object->" ) + ptype + "->functions->function" + number + "->return", "" );
			std::cout << "\t\t" << ( virtual_state=="no"?"":"virtual " ) << ret << " " << name << "(" << args << " )"<< ( virtual_state=="pure"?" = 0":"" ) <<";" << std::endl;
		}

	}
}
void generate_accessor ( const std::string &prefix, const std::string &accessor_type, const std::string &variable_type, const std::string &variable_name )
{
	if ( accessor_type.find ( "r" ) !=std::string::npos ) accessors_list += prefix + variable_type + std::string(" _get_")+variable_name + std::string("() const { return ")+variable_name+std::string(";}\n");
	if ( accessor_type.find ( "w" ) !=std::string::npos ) accessors_list += prefix + std::string("void _set_")+variable_name+std::string("(")+variable_type+std::string(" t_")+variable_name+std::string(") { ")+variable_name+std::string(" = t_")+variable_name+std::string(";}\n");
}
void write_input_params ( const std::string &class_name, std::vector<std::string> &base_classes, Config_Interface *intf )
{
	std::cout << "// loadable params" << std::endl;
	char number[ 7 ];
	//Загрузим переменные
	for ( int a = 0;a < 999999; ++a )
	{
		snprintf ( number, 7, "%d", a );
		std::string name = intf->_get_value ( std::string ( "object->input_params->param" ) + number + "->name", "" );
		std::string type = intf->_get_value ( std::string ( "object->input_params->param" ) + number + "->type", "" );
		std::string accessor_type = intf->_get_value ( std::string ( "object->input_params->param" ) + number + "->accessor_type", "rw" );
		if ( name.empty() ) break;

		if ( type == "text" )
		{
			std::cout << "\t\t" << "std::string " << name << ";" << std::endl;
			generate_accessor ( "\t\t", accessor_type, "std::string", name );
		}
		else if ( type == "float" )
		{
			std::cout << "\t\t" << "float " << name << ";" << std::endl;
			generate_accessor ( "\t\t", accessor_type, "float", name );
		}
		else if ( type == "int" )
		{
			std::cout << "\t\t" << "int " << name << ";" << std::endl;
			generate_accessor ( "\t\t", accessor_type, "int", name );
		}
		else if ( type == "unsigned int" )
		{
			std::cout << "\t\t" << "unsigned int " << name << ";" << std::endl;
			generate_accessor ( "\t\t", accessor_type, "unsigned int", name );
		}
		else if ( type == "bool" )
		{
			std::cout << "\t\t" << "bool " << name << ";" << std::endl;
			generate_accessor ( "\t\t", accessor_type, "bool", name );
		}

	}
}
void write_variables ( const std::string &class_name, std::vector<std::string> &base_classes, Config_Interface *intf, const std::string ptype )
{
	std::cout << "//" << ptype << " variables" << std::endl;
	char number[ 7 ];
	//Загрузим переменные
	for ( int a = 0;a < 999999; ++a )
	{
		snprintf ( number, 7, "%d", a );
		std::string name = intf->_get_value ( std::string ( "object->" ) + ptype + "->variables->variable" + number + "->name", "" );
		std::string type = intf->_get_value ( std::string ( "object->" ) + ptype + "->variables->variable" + number + "->type", "" );
		std::string accessor_type = intf->_get_value ( std::string ( "object->" ) + ptype + "->variables->variable" + number + "->accessor_type", "rw" );
		if ( name.empty() ) break;
		std::cout << "\t\t" << type << " " << name << ";" << std::endl;
		generate_accessor ( "\t\t", accessor_type, type, name );

	}

}
void write_structs ( const std::string &class_name, std::vector<std::string> &base_classes, Config_Interface *intf, const std::string ptype )
{
	std::cout << "//" << ptype << " structs definitions" << std::endl;
	char number[ 7 ];
	//Загрузим переменные
	for ( int a = 0;a < 999999; ++a )
	{
		snprintf ( number, 7, "%d", a );
		std::string name = intf->_get_value ( std::string ( "object->" ) + ptype + "->structs->struct" + number + "->declaration", "" );
		std::string type = intf->_get_value ( std::string ( "object->" ) + ptype + "->structs->struct" + number + "->type", "" );
		if ( name.empty() ) break;
		std::cout << "\t\t" << type << " " << name << ";" << std::endl;
	}

}
void define_slot ( const std::string &class_name, std::vector<std::string> &base_classes, Config_Interface *intf, const std::string slot_path )
{
	std::string name = intf->_get_value ( slot_path + "->name", "" );
	std::string type = intf->_get_value ( slot_path + "->type", "" );
	if ( name.empty() ) return;

	std::cout << "\t\t{" << std::endl;
	if ( name == "_on_activate" || name == "_on_deactivate" ) std::cout << "\t\t\treturn " << name << "_handler();" << std::endl;
	else if ( name == "_on_key_press" || name == "_on_key_release" )
	{
		LINE ( "\t\t\t", "int64_t keyboard_state;" );
		LINE ( "\t\t\t", "try" );
		LINE ( "\t\t\t", "{" );
		LINE ( "\t\t\t\t", "keyboard_state = args->int_args.at(0);" );
		LINE ( "\t\t\t", "}" );
		LINE ( "\t\t\t", "catch(...){return true;};" );
		std::cout << "\t\t\treturn " << name << "_handler(keyboard_state);" << std::endl;
	}
	else if ( name == "_on_character" )
	{
		LINE ( "\t\t\t", "int64_t character;" );
		LINE ( "\t\t\t", "try" );
		LINE ( "\t\t\t", "{" );
		LINE ( "\t\t\t\t", "character = args->int_args.at(0);" );
		LINE ( "\t\t\t", "}" );
		LINE ( "\t\t\t", "catch(...){return true;};" );
		std::cout << "\t\t\treturn " << name << "_handler(character);" << std::endl;
	}
	else if ( name == "_on_mouse_press" || name == "_on_mouse_release" || name == "_on_mouse_move" )
	{
		LINE ( "\t\t\t", "float x_pos;" );
		LINE ( "\t\t\t", "float y_pos;" );
		LINE ( "\t\t\t", "float dx;" );
		LINE ( "\t\t\t", "float dy;" );
		LINE ( "\t\t\t", "int64_t button_state;" );
		LINE ( "\t\t\t", "try" );
		LINE ( "\t\t\t", "{" );
		LINE ( "\t\t\t\t", "button_state= args->int_args.at(0);" );
		LINE ( "\t\t\t\t", "x_pos = args->float_args.at(0);" );
		LINE ( "\t\t\t\t", "y_pos = args->float_args.at(1);" );
		LINE ( "\t\t\t\t", "dx = args->float_args.at(2);" );
		LINE ( "\t\t\t\t", "dy = args->float_args.at(3);" );
		LINE ( "\t\t\t", "}" );
		LINE ( "\t\t\t", "catch(...){return true;};" );
		std::cout << "\t\t\treturn " << name << "_handler(x, y, dx, dy, button_state);" << std::endl;
	}
	else //Кастомный слот
	{
		//Получим параметры слота
		char num[ 7 ];
		for ( int b = 0;b < 999999; ++b )
		{
			snprintf ( num, 7, "%d", b );
			std::string spname = intf->_get_value ( slot_path + "->slot_params->param" + num + "->name", "" );
			std::string sptype = intf->_get_value ( slot_path + "->slot_params->param" + num + "->type", "" );
			if ( spname.empty() ) break;

			if ( sptype == "text" ) sptype = "std::string";
			else if ( sptype == "int" ) sptype = "int64_t";
			std::cout << "\t\t\t" << sptype << " " << spname << ";" << std::endl;
		}
		if ( !intf->_get_value ( slot_path + "->slot_params->param0->name", "" ).empty() )
		{
			LINE ( "\t\t\t", "try" );
			LINE ( "\t\t\t", "{" );
			int f = 0, s = 0, i = 0;
			for ( int b = 0;b < 999999; ++b )
			{
				snprintf ( num, 7, "%d", b );
				std::string spname = intf->_get_value ( slot_path + "->slot_params->param" + num + "->name", "" );
				std::string sptype = intf->_get_value ( slot_path + "->slot_params->param" + num + "->type", "" );
				if ( spname.empty() ) break;

				if ( sptype == "text" ) std::cout << "\t\t\t" << spname << " = " << "args->string_args.at(" << s++ << ");" << std::endl;
				else if ( sptype == "int" ) std::cout << "\t\t\t" << spname << " = " << "args->int_args.at(" << i++ << ");" << std::endl;
				else if ( sptype == "float" ) std::cout << "\t\t\t" << spname << " = " << "args->float_args.at(" << f++ << ");" << std::endl;
			}
			LINE ( "\t\t\t", "}" );
			LINE ( "\t\t\t", "catch(...){return true;};" );
		}
		std::cout << "\t\t\treturn " << name << "_handler(";
		//Получим параметры слота
		for ( int b = 0;b < 999999; ++b )
		{
			snprintf ( num, 7, "%d", b );
			std::string spname = intf->_get_value ( slot_path + "->slot_params->param" + num + "->name", "" );
			std::string sptype = intf->_get_value ( slot_path + "->slot_params->param" + num + "->type", "" );
			if ( spname.empty() ) break;

			if ( sptype == "text" ) sptype = "std::string";
			else if ( sptype == "int" ) sptype = "int64_t";
			std::cout << ( b ? ", " : " " ) << spname;
		}
		std::cout << ");" << std::endl;
	}
	std::cout << "\t\t}" << std::endl;
}

void write_slots ( const std::string &class_name, std::vector<std::string> &base_classes, Config_Interface *intf, const std::string ptype )
{
	std::cout << "//" << ptype << " slots" << std::endl;
	char number[ 7 ];
	//Загрузим слоты
	for ( int a = 0;a < 999999; ++a )
	{
		snprintf ( number, 7, "%d", a );
		std::string name = intf->_get_value ( std::string ( "object->" ) + ptype + "->slots->slot" + number + "->name", "" );
		std::string type = intf->_get_value ( std::string ( "object->" ) + ptype + "->slots->slot" + number + "->type", "" );
		std::string define = intf->_get_value ( std::string ( "object->" ) + ptype + "->slots->slot" + number + "->define", "true" );
		if ( name.empty() ) break;
		if ( type != "virtual" ) std::cout << "\t\tDECLARE_SLOT(" << class_name << ", " << name << " )";
		else std::cout << "\t\tVIRTUAL_SLOT(" << name << " )";

		if ( define!="true" )
		{
			std::cout<<";"<<std::endl;
			continue;
		}
		else std::cout<<std::endl;
		define_slot ( class_name, base_classes, intf, std::string ( "object->" ) + ptype + "->slots->slot" + number );

		std::cout << "\t\tbool " << name << "_handler";
		if ( name == "_on_activate" ) std::cout << "();" << std::endl;
		else if ( name == "_on_deactivate" ) std::cout << "();" << std::endl;
		else if ( name == "_on_key_press" ) std::cout << "(int64_t keyboard_state);" << std::endl;
		else if ( name == "_on_key_release" ) std::cout << "(int64_t keyboard_state);" << std::endl;
		else if ( name == "_on_character" ) std::cout << "(int64_t character);" << std::endl;
		else if ( name == "_on_mouse_press" ) std::cout << "(float x_pos, float y_pos, float dx, float dy, int64_t button_state);" << std::endl;
		else if ( name == "_on_mouse_release" ) std::cout << "(float x_pos, float y_pos, float dx, float dy, int64_t button_state);" << std::endl;
		else if ( name == "_on_mouse_move" ) std::cout << "(float x_pos, float y_pos, float dx, float dy, int64_t button_state);" << std::endl;
		else //Кастомный слот
		{
			std::cout << "(";
			//Получим параметры слота
			char num[ 7 ];
			for ( int b = 0;b < 999999; ++b )
			{
				snprintf ( num, 7, "%d", b );
				std::string spname = intf->_get_value ( std::string ( "object->" ) + ptype + "->slots->slot" + number + "->slot_params->param" + num + "->name", "" );
				std::string sptype = intf->_get_value ( std::string ( "object->" ) + ptype + "->slots->slot" + number + "->slot_params->param" + num + "->type", "" );
				if ( spname.empty() ) break;

				if ( sptype == "text" ) sptype = "std::string";
				else if ( sptype == "int" ) sptype = "int64_t";
				std::cout << ( b ? ", " : " " ) << sptype << " " << spname;
			}
			std::cout << ");" << std::endl;
		}
	}

}
void write_signals ( const std::string &class_name, std::vector<std::string> &base_classes, Config_Interface *intf, const std::string ptype )
{
	std::cout << "//" << ptype << " signals" << std::endl;
	char number[ 7 ];
	//Загрузим слоты
	for ( int a = 0;a < 999999; ++a )
	{
		snprintf ( number, 7, "%d", a );
		std::string name = intf->_get_value ( std::string ( "object->" ) + ptype + "->signals->signal" + number + "->name", "" );
		if ( name.empty() ) break;
		std::cout << "\t\tSignal " << name << ";" << std::endl;
		std::cout << "\t\tvoid rise_" << name << "(";

		char num[ 7 ];
		for ( int b = 0;b < 999999; ++b )
		{
			snprintf ( num, 7, "%d", b );
			std::string spname = intf->_get_value ( std::string ( "object->" ) + ptype + "->signals->signal" + number + "->signal_params->param" + num + "->name", "" );
			std::string sptype = intf->_get_value ( std::string ( "object->" ) + ptype + "->signals->signal" + number + "->signal_params->param" + num + "->type", "" );
			if ( spname.empty() ) break;

			std::cout << ( b ? ", " : " " );

			if ( sptype == "text" ) std::cout << "std::string " << spname;
			else if ( sptype == "int" ) std::cout << "int64_t " << spname;
			else if ( sptype == "float" ) std::cout << "float " << spname;
			else
			{
				std::cerr << "Ivalid signal parameter type: " << sptype << std::endl;
				exit ( 1 );
			}
		}
		std::cout << ")" << std::endl;

		std::cout << "\t\t{" << std::endl;
		std::cout << "\t\t\t" << name << ".clear_params();" << std::endl;
		for ( int b = 0;b < 999999; ++b )
		{
			snprintf ( num, 7, "%d", b );
			std::string spname = intf->_get_value ( std::string ( "object->" ) + ptype + "->signals->signal" + number + "->signal_params->param" + num + "->name", "" );
			std::string sptype = intf->_get_value ( std::string ( "object->" ) + ptype + "->signals->signal" + number + "->signal_params->param" + num + "->type", "" );
			if ( spname.empty() ) break;
			std::cout << "\t\t\t" << name << ".add_params(" << spname << ");" << std::endl;
		}
		std::cout << "\t\t\t" << name << ".rise();" << std::endl;
		std::cout << "\t\t}" << std::endl;
	}
}
void generate_header ( char *config_name )
{
	Config_Interface * intf = config_get_interface ( config_name );
	if ( !intf )
	{
		std::cerr << "Error with config";
		return ;
	}

	std::cout << header;
	write_includes ( intf );
	//Распарсим список наследования и заинклудим требуемые хедеры
	char number[ 7 ];
	for ( int a = 0;a < 999999; ++a )
	{
		snprintf ( number, 7, "%d", a );
		std::string name = intf->_get_value ( std::string ( "object->base->base_class" ) + number + "->name", "" );
		std::string inheritance = intf->_get_value ( std::string ( "object->base->base_class" ) + number + "->inheritance", "public" );
		if ( name.empty() ) break;
		std::transform ( name.begin(), name.end(), name.begin(), tolower );
		std::cout << "#include \"" << name << ".h\"" << std::endl;
	}

	std::cout << std::endl;
	std::string class_name = intf->_get_value ( "object->name", "" );
	std::cout << "class " << class_name;
	std::vector<std::string> base_classes;
	//Распарсим список наследования
	for ( int a = 0;a < 999999; ++a )
	{
		snprintf ( number, 7, "%d", a );
		std::string name = intf->_get_value ( std::string ( "object->base->base_class" ) + number + "->name", "" );
		std::string inheritance = intf->_get_value ( std::string ( "object->base->base_class" ) + number + "->inheritance", "public" );
		if ( name.empty() ) break;
		base_classes.push_back ( name );
		std::cout << ( ( a == 0 ) ? " : " : ", " ) << "virtual " << inheritance << " " << name;
	}
	std::cout << std::endl << "{" << std::endl;

	//Распарсим список функций public блока
	std::cout << "\tpublic:" << std::endl;
	write_slots ( class_name, base_classes, intf, std::string ( "public" ) );
	write_functions ( class_name, base_classes, intf, std::string ( "public" ) );
	write_signals ( class_name, base_classes, intf, std::string ( "public" ) );

	std::cout << "\tprotected:" << std::endl;
	write_slots ( class_name, base_classes, intf, std::string ( "protected" ) );
	write_functions ( class_name, base_classes, intf, std::string ( "protected" ) );
	write_signals ( class_name, base_classes, intf, std::string ( "protected" ) );

	std::cout << "\tprivate:" << std::endl;
	write_slots ( class_name, base_classes, intf, std::string ( "private" ) );
	write_functions ( class_name, base_classes, intf, std::string ( "private" ) );
	write_signals ( class_name, base_classes, intf, std::string ( "private" ) );


	std::cout << "\tpublic:" << std::endl;
	write_structs ( class_name, base_classes, intf, std::string ( "public" ) );
	write_variables ( class_name, base_classes, intf, std::string ( "public" ) );
	write_input_params ( class_name, base_classes, intf );

	std::cout << "\tprotected:" << std::endl;
	write_structs ( class_name, base_classes, intf, std::string ( "protected" ) );
	write_variables ( class_name, base_classes, intf, std::string ( "protected" ) );

	std::cout << "\tprivate:" << std::endl;
	write_structs ( class_name, base_classes, intf, std::string ( "private" ) );
	write_variables ( class_name, base_classes, intf, std::string ( "private" ) );

	std::cout<<"//Accessors"<<std::endl;
	std::cout<<"\tpublic:"<<std::endl;
	std::cout<<accessors_list<<std::endl;

	std::cout << "};" << std::endl;

	std::cout << std::endl;
	std::string fabric = intf->_get_value ( "object->fabric", "true" );
	if ( fabric!="false" )
	{
		std::cout << "extern ObjectFabric<" << class_name << "> " << class_name << "Fabric;" << std::endl;
		std::cerr << "SceneObjectFabric::add_fabric ( \"" << class_name << "\", &" << class_name << "Fabric );" << std::endl;
	}

}


int main ( int argc, char *argv[] )
{
	if ( argc < 2 )
	{
		std::cout << "Usage: " << argv[ 0 ] << " <input file>" << std::endl;
		return 1;
	}
	FileIO::FileManager *filemanager;
	filemanager = new FileIO::FileManager ( 100 );
	filemanager->register_fileio ( &FileIO::CreatePosixFile );
	if ( !load_config ( argv[ 1 ], ( char* ) "Object", filemanager ) ) return 1;
	generate_header ( ( char* ) "Object" );

	delete filemanager;
	return 0;
}
