//
// C++ Implementation: numeric
//
// Description:
//
//
// Author: Messenger of death aka Denis A. Birukov <messengerofdeath@gmail.com>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "numeric.h"
#include "video.h"
#include "video_int.h"
#include "event.h"
#include "textrenderer.h"
#include "signal_manager.h"
#include <stdio.h>

ObjectFabric<Numeric> NumericFabric;

void Numeric::draw()
{
    if ( is_value_changed )
    {
        char buf[value_length+2];
        snprintf( buf, value_length + 1, value_spec, value );
        text = buf;
        Video_Interface *vid = video_get_interface( _video_name_ );
        if ( !vid )
        {
            _set_error( "Can't get video interface:" + video_error() );
            return;
        }

        Text_Renderer *renderer;
        renderer = textrenderer_get( renderer_type );
        if ( !renderer )
        {
            _set_error( "Can't get renderer of type \"" + renderer_type + " :" + textrenderer_error() );
            return;
        }
	VideoColor clr;
	clr.set_color( color );
        if ( !renderer->render_text_line( _video_name_, font, text, clr, font_size, _object_name_, _scene_name_, true ) )
        {
            _set_error( "Can't render text:" + renderer->_get_error() );
            return;
        }
        if ( !vid->_create_sprite( _object_name_, _scene_name_, _object_name_, _scene_name_, true ) )
        {
            _set_error( "Can't create sprite: " + vid->_get_error() );
            return;
        }

        Video_Sprite *spr = vid->_get_sprite( _object_name_, _scene_name_ );
        if ( !spr )
        {
            _set_error( "Can't get sprite: " + vid->_get_error() );
            return;
        }
        spr->_set_base_point( x_base, y_base );
        spr->_set_coord( x, y );
        is_value_changed = false;
    }
    Text::draw();

}

bool Numeric::real_load( Scene *scene, std::string storage_name, std::string storage_prefix, std::string scene_name, std::string object_name, FileIO::FileManager *fmanager )
{
	value_spec = new char[10];
    snprintf( value_spec, 9, "%%0%dd", value_length );
    value = start_value;
    is_value_changed = true;
    return true;
}

bool Numeric::increment_value_handler()
{
    value++;
    is_value_changed = true;
    return false;
}
bool Numeric::decrement_value_handler()
{
    value--;
    is_value_changed = true;
    return false;
}
bool Numeric::add_value_handler(int64_t new_value)
{
    value += new_value;
    is_value_changed = true;
    return false;
}
bool Numeric::sub_value_handler(int64_t new_value)
{
	value -= new_value;
	is_value_changed = true;
	return false;
}
bool Numeric::set_value_handler(int64_t new_value)
{
	value = new_value;
	is_value_changed = true;
	return false;
}
Numeric::~Numeric()
{
	if(value_spec)delete[] value_spec;
}
