//
// C++ Implementation: scene_object
//
// Description:
//
//
// Author: Denis A. Birukov <denis.birukov@gmail.com>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "sceneobject.h"
#include "video_int.h"
#include "video.h"
#include "signal_manager.h"
#include <stdio.h>
#include "scene.h"

bool SceneObject::real_load ( Scene *scene, std::string storage_name, std::string storage_prefix, std::string scene_name, std::string object_name, FileIO::FileManager *fmanager )
{
	Config_Interface *intf = config_get_interface ( storage_name );
	if ( !intf )
	{
		_set_error ( "Can't get config interface:" + config_error() );
		return false;
	}
	if ( coord_type == "absolute" )
	{
		Scene *parent = scene->get_parent();
		if(parent)
		{
			x = (absolute_x * scene->get_width())/parent->get_width() + scene->get_x();
			y = (absolute_y * scene->get_height())/parent->get_height() + scene->get_y();
		}
		else
		{
			x = absolute_x;
			y = absolute_y;
		}
	}
	else if ( coord_type == "link" )
	{
		Video_Interface *vid = video_get_interface ( _video_name_ );
		if ( !vid )
		{
			_set_error ( "Can't get video interface:" + video_error() );
			return false;
		}

		Video_Sprite *spr;
		spr = vid->_get_sprite ( link_object, _scene_name_ );
		if ( !spr )
		{
			_set_error ( "Can't get sprite: " + vid->_get_error() );
			return false;
		}
		
		VideoRect scoord;
		spr->_get_rectangle ( &scoord );
		if ( link_v_align == "top" ) y = scoord.y_e;
		else if ( link_v_align == "bottom" ) y = scoord.y_s;
		else if ( link_v_align == "center" ) y = scoord.y_s + ( scoord.y_e - scoord.y_s ) / 2.;
		else if ( link_v_align == "base_point" ) y = spr->_get_y() + spr->_get_base_y();
		else
		{
			_set_error ( "Unknown v_align kind: \"" + link_v_align + "\"" );
			return false;
		}
		y += spr->_get_v_offset ( link_v_offset );
		if ( link_h_align == "left" ) x = scoord.x_s;
		else if ( link_h_align == "right" ) x = scoord.x_e;
		else if ( link_h_align == "center" ) x = scoord.x_s + ( scoord.x_e - scoord.x_s ) / 2.;
		else if ( link_h_align == "base_point" ) x = spr->_get_x() + spr->_get_base_x();
		else
		{
			_set_error ( "Unknown h_align kind: \"" + link_h_align + "\"" );
			return false;
		}
		x += spr->_get_h_offset ( link_h_offset );
	}
	else
	{
		_set_error ( "Unknown coord kind: " + coord_type );
		return false;
	}
	load_signals ( intf, storage_prefix );
	return true;
};
void SceneObject::_get_active_area_rect ( VideoRect * rect )
{
	rect->x_s = 0;
	rect->y_s = 0;
	rect->x_e = 0;
	rect->y_e = 0;
	Video_Interface *vid = video_get_interface ( _video_name_ );
	if ( !vid ) return;
	Video_Sprite *spr = vid->_get_sprite ( _object_name_, _scene_name_ );
	if ( !spr ) return;
	spr->_get_rectangle ( rect );
}

void SceneObject::load_signals ( Config_Interface *intf, std::string storage_prefix )
{
	char number[9];
	std::string prefix = storage_prefix + "signals->signal";
	register_slots();
	//Добавим дефолтные сигналы
	SignalName signal_name_t = _scene_name_ + "->activate";
	SlotName slot_name_t = _scene_name_ + "->" + _object_name_ + "->_on_activate";
	_signal_connections_.push_back ( SignalConnection ( signal_name_t, slot_name_t, false ) );

	signal_name_t = _scene_name_ + "->deactivate";
	slot_name_t = _scene_name_ + "->" + _object_name_ + "->_on_deactivate";
	_signal_connections_.push_back ( SignalConnection ( signal_name_t, slot_name_t, false ) );

	signal_name_t = _scene_name_ + "->reinit";
	slot_name_t = _scene_name_ + "->" + _object_name_ + "->_on_scene_reinit";
	_signal_connections_.push_back ( SignalConnection ( signal_name_t, slot_name_t, false ) );
	
	for ( int a = 0;a < 1000000;a++ )
	{
		snprintf ( number, 8, "%d->", a );
		std::string connect_type = intf->_get_value ( prefix + number + "connect", "on_request" );
		std::string type = intf->_get_value ( prefix + number + "type", "custom" );
		std::string signal_source = intf->_get_value ( prefix + number + "signal_source", "" );
		if ( signal_source.length() == 0 ) break;
		std::string signal_name = intf->_get_value ( prefix + number + "signal_name", "" );
		std::string slot_name = intf->_get_value ( prefix + number + "slot_name", "" );

		if ( signal_name == "activate" || signal_name == "deactivate" || !signal_name.length() ) continue;
		if ( type == "builtin" )
		{
			if ( signal_source == "Video" ) signal_source = _video_name_;
		}
		bool is_connect_on_load = (connect_type=="on_load");
		signal_name_t = signal_source + "->" + signal_name;
		slot_name_t = _scene_name_ + "->" + _object_name_ + "->" + slot_name;
		_signal_connections_.push_back ( SignalConnection ( signal_name_t, slot_name_t, is_connect_on_load ) );


		//Если режим connect: on_load сразу производим соединение сигнала
		if ( connect_type == "on_load" ) connect ( signal_name_t, slot_name_t );
	}
}
void SceneObject::disconnect_signals(bool is_disconnect_on_load)
{
	SignalName signal_name_t = _scene_name_ + "->reinit";
	SlotName slot_name_t = _scene_name_ + "->" + _object_name_ + "->_on_scene_reinit";
	disconnect (signal_name_t, slot_name_t);
	
	///@TODO вернуть auto на место
	std::vector<SignalConnection>::iterator it;
	for ( it = _signal_connections_.begin(); it != _signal_connections_.end(); ++it )
	{
		if(!is_disconnect_on_load && it->is_connect_on_load)continue;
		disconnect ( it->signal_name, it->slot_name );
	}
}

void SceneObject::connect_signals(bool on_load)
{
	for ( std::vector<SignalConnection>::iterator it = _signal_connections_.begin(); it != _signal_connections_.end(); ++it )
	{
		if(on_load)
		{
			if(it->is_connect_on_load)connect ( it->signal_name, it->slot_name );
		}
		else connect ( it->signal_name, it->slot_name );
	}
}
void SceneObject::register_slots()
{
	for(std::vector<std::pair<std::string, AbstractSlot*> >::iterator it=_slot_map_.begin(); it!=_slot_map_.end(); ++it)
	{
		register_slot( it->first, it->second);
	}
	
	SignalName signal_name_t = _scene_name_ + "->reinit";
	SlotName slot_name_t = _scene_name_ + "->" + _object_name_ + "->_on_scene_reinit";
	connect (signal_name_t, slot_name_t);
}

void SceneObject::unregister_slots()
{
	for(std::vector<std::pair<std::string, AbstractSlot*> >::iterator it=_slot_map_.begin(); it!=_slot_map_.end(); ++it)
	{
		unregister_slot( it->first);
	}
	
	SignalName signal_name_t = _scene_name_ + "->reinit";
	SlotName slot_name_t = _scene_name_ + "->" + _object_name_ + "->_on_scene_reinit";
	disconnect (signal_name_t, slot_name_t);
}
DEFINE_SLOT(SceneObject, _on_activate )
{
	return true;
}
DEFINE_SLOT(SceneObject, _on_deactivate )
{
	return true;
}
DEFINE_SLOT(SceneObject, _on_key_press )
{
	return true;
}
DEFINE_SLOT(SceneObject, _on_key_release )
{
	return true;
}
DEFINE_SLOT(SceneObject, _on_character )
{
	return true;
}
DEFINE_SLOT(SceneObject, _on_mouse_release )
{
	return true;
}
DEFINE_SLOT(SceneObject, _on_scene_reinit )
{
	connect_signals(true);
	return false;
}
std::string SceneObject::_get_name(void )
{
	return _video_name_;
}
void SceneObject::set_video_name(std::string video_name )
{
	_video_name_ = video_name;
}
std::string SceneObject::_get_error(void )
{
	return _error_;
}
void SceneObject::_set_error(std::string error )
{
	
	_error_ = error;
}
