//
// C++ Implementation: textedit
//
// Description:
//
//
// Author: Denis A. Birukov <denis.birukov@gmail.com>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "textedit.h"
#include "video.h"
#include "video_int.h"
#include "event.h"
#include "textrenderer.h"
#include "signal_manager.h"

ObjectFabric<TextEdit> TextEditFabric;

void TextEdit::real_draw()
{
    if ( text.empty() ) return;
    std::vector<int> replace_string;
    replacer = *(replace_text.c_str());
    if ( !replace_text.empty() )
    {
	    for ( unsigned int a = 0;a < text.size();a++ ) replace_string.push_back(replacer);
    }

    Video_Interface *vid = video_get_interface ( _video_name_ );
    if ( !vid )
    {
        _set_error ( "Can't get video interface:" + video_error() );
        return;
    }

    Text_Renderer *renderer = textrenderer_get ( renderer_type );
    if ( !renderer )
    {
        _set_error ( "Can't get renderer:" + textrenderer_error() );
        return;
    }

    VideoColor clr;
    clr.set_color(color);
    if ( !renderer->render_text_line_to_sprite ( _video_name_, font, ( replace_text.empty() ) ? text : replace_string, clr, font_size, _object_name_ + "txt", _scene_name_, true ) )
    {
        _set_error ( "Can't render text:" + renderer->_get_error() );
        return;
    }

    Video_Sprite *spr = vid->_get_sprite ( _object_name_ + "txt", _scene_name_ );
    if ( !spr )
    {
        _set_error ( "Can't get sprite: " + vid->_get_error() );
        return;
    }
    Video_Sprite *img = vid->_get_sprite ( _object_name_ , _scene_name_ );
    if ( !img )
    {
        _set_error ( "Can't get sprite: " + vid->_get_error() );
        return;
    }
    spr->_set_base_point ( x_base, y_base );
    spr->_set_coord ( img->_get_x() - img->_get_base_x(), img->_get_y() - img->_get_base_y() );
    vid->_add_sprite ( _object_name_ + "txt", _scene_name_ );
}

bool TextEdit::_on_activate_handler()
{
    Video_Interface *vid = video_get_interface ( _video_name_ );
    if ( !vid )
    {
        _set_error ( "Can't get video interface:" + video_error() );
        return false;
    }
    Video_Sprite *spr = vid->_get_sprite ( _object_name_, _scene_name_ );
    if ( !spr )
    {
        _set_error ( "Can't get sprite: " + vid->_get_error() );
        return false;
    }
    vid->_enable_char_event(true);
	vid->_enable_keyboard_event(true);
    spr->_set_blend_color ( 0.5, 0.5, 0.5 );
    SignalName signal_name_t = _video_name_+"->character";
    SlotName slot_name_t = _scene_name_ + "->" + _object_name_ + "->_on_character";
    connect(signal_name_t, slot_name_t);
    
    signal_name_t = _video_name_ + "->keyboard_release";
    slot_name_t = _scene_name_ + "->" + _object_name_ + "->_on_key_press";
    connect(signal_name_t, slot_name_t);

    return false;
}
bool TextEdit::_on_deactivate_handler()
{
    Video_Interface *vid = video_get_interface ( _video_name_ );
    if ( !vid )
    {
        _set_error ( "Can't get video interface:" + video_error() );
        return false;
    }
    Video_Sprite *spr = vid->_get_sprite ( _object_name_, _scene_name_ );
    if ( !spr )
    {
        _set_error ( "Can't get sprite: " + vid->_get_error() );
        return false;
    }
    spr->_set_blend_color ( 1.0, 1.0, 1.0 );
    
    SignalName signal_name_t = _video_name_+"->character";
    SlotName slot_name_t = _scene_name_ + "->" + _object_name_ + "->_on_character";
    disconnect(signal_name_t, slot_name_t);
    
    signal_name_t = _video_name_+"->activate";
    slot_name_t = _scene_name_ + "->" + _object_name_ + "->_on_activate";
    disconnect(signal_name_t, slot_name_t);
    
    signal_name_t = _video_name_ + "->keyboard_release";
    slot_name_t = _scene_name_ + "->" + _object_name_ + "->_on_key_press";
    disconnect(signal_name_t, slot_name_t);
    return true;
}

bool TextEdit::_on_character_handler(int64_t character)
{
    if ( text.size() < (unsigned)max_length ) text.push_back ( character );
    return true;
}

bool TextEdit::_on_key_press_handler(int64_t keyboard_state)
{
    switch(keyboard_state)
    {
        case KEY_BACKSPACE: 
	{ 
		if(!text.empty())text.pop_back();
		break;
	}
        default:return true;
    }
    return true;
}

