#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "scenemanager.h"
#include "scene.h"
#include <QFileDialog>
#include "sceneobject.h"

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent), ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    fonts = new Fonts;
    filemanager = new FileIO::FileManager(100);
    filemanager->register_fileio(&FileIO::CreatePosixFile);
    ui->ObjectParamsWidget->set_fmanager(filemanager);

    fonts->set_fileio(filemanager);

    thread = new MyThread;
    thread->filemanager = filemanager;
    thread->start();

    ui->AddedObjects->setColumnWidth(0, 200);
    ui->AddedObjects->setColumnWidth(1, 40);
}

MainWindow::~MainWindow()
{
    delete ui;
    delete filemanager;
}

bool MyThread::init()
{
        if ( !imageloader_create ( "jpg", "libloader_jpg", true ) )
        {
                std::cerr<<"Failed to load JPEG loader \"libloader_jpg\""<<imageloader_error() <<std::endl;
                return false;
        }
        if ( !imageloader_create ( "png", "libloader_png", true ) )
        {
                std::cerr<<"Failed to load PNG loader \"libloader_png\""<<imageloader_error() <<std::endl;
                return false;
        }
        if ( !textrenderer_create ( "ttf", "libttf_renderer", true ) )
        {
                std::cerr<<"Failed to load TTF renderer \"librenderer_ttf\""<<textrenderer_error() <<std::endl;
                return false;
        }

        Config_Interface *intf;
        if ( !config_create ( "VideoConfig", "libtextconfig" ) )
        {
                std::cerr<< "Error while trying to create config from libtextconfig: " << config_error() << std::endl;
                return false;
        }
        intf = config_get_interface ( "VideoConfig" );
        if ( !intf )
        {
                std::cerr<< "Error with config";
                return false ;
        }
        intf->_load ( "cfg/Video.cfg", filemanager );
        std::string drv = intf->_get_value ( "Video->Driver", "libGL_glfw" );
        std::cerr<< "Video driver: " << drv << std::endl;

        std::string params1[ 3 ];
        params1[ 0 ] = "VideoConfig";
        params1[ 1 ] = "Video->";
        params1[ 2 ] = "ME_Video";

        if ( !video_create ( "Video", drv, params1, 3 ) )
        {
                std::cerr<< "Error while trying to create video from " << drv << ":" << video_error() << std::endl;
                return false;
        }
        Video_Interface *vid = video_get_interface ( "Video" );
        if ( !vid ) return false;
        if ( !vid->_init() )
        {
                std::cerr<< "Error while trying to initialyze video: " << vid->_get_error() << std::endl;
                return false;
        }
        ::init(std::string("Video"));
        return true;
}
void MyThread::run()
{
    if(!init())return;
    load_scenes ( "res/scenes.scns", "Video", filemanager );

    process_scene ( "empty_scene" );

    Video_Interface *vid = video_get_interface ( "Video" );
    vid->_deinit();
    return;
}

void MainWindow::on_actionLoad_object_set_triggered()
{
        QFileDialog dialog;
        dialog.setDefaultSuffix ( "object" );
        dialog.setNameFilter ( tr ( "Object description (*.object)" ) );
        dialog.setLabelText ( QFileDialog::Accept, tr ( "Load object" ) );
        dialog.setFileMode ( QFileDialog::ExistingFiles );
        dialog.setAcceptMode ( QFileDialog::AcceptOpen);
        dialog.setViewMode ( QFileDialog::Detail );
        if ( dialog.exec() )
        {
            QStringList selectedFiles = dialog.selectedFiles();
            QString loadFileName;
            foreach (loadFileName, selectedFiles)
            {
                config_create ( loadFileName.toUtf8().data(), "libtextconfig" );
                Config_Interface *intf = config_get_interface ( loadFileName.toUtf8().data() );
                if ( !intf ){std::cerr<< "Error with config";return;}

                intf->_load(loadFileName.toUtf8().data(), filemanager);
                std::string oname = intf->_get_value("object->name","");

                config_map.insert(make_pair(oname, std::string(loadFileName.toUtf8().data())));

                ui->ObjectList->addItem(oname.c_str());
            }
        }
}
bool MainWindow::fill_object_config(QString object_name, QString class_name)
{
    if(!config_create(object_name.toUtf8().data(), "libtextconfig"))
    {
        std::cerr<<"Unable to create config:"<<object_name.toUtf8().data()<<". "<<config_error()<<std::endl;
        return false;
    }
    Config_Interface *ocfg = config_get_interface(object_name.toUtf8().data());
    Config_Interface *object_class = config_get_interface(config_map.find(class_name.toUtf8().data())->second);
    if(!ocfg || !object_class)
    {
        std::cerr<<__LINE__<<"Unable to get config interface:"<<config_error()<<std::endl;
        return false;
    }
    //For each object base class
    char number[7];
    for(int a=0;a<999999;++a)
    {
        snprintf(number, 7, "%d", a);
        std::string bname = object_class->_get_value(std::string("object->base->base_class")+number+"->name", "");
        if(bname.empty())break;
        Config_Interface *base = config_get_interface(config_map.find(bname)->second);
        if(!base)
        {
            std::cerr<<__LINE__<<"Unable to get config interface:"<<config_error()<<std::endl;
            return false;
        }
        char number1[7];
        for(int b=0;b<999999;++b)
        {
            snprintf(number1, 7, "%d", b);
            std::string pname = base->_get_value(std::string("object->input_params->param")+number1+"->name", "");
            std::string pdef = base->_get_value(std::string("object->input_params->param")+number1+"->default", "");
            std::string ptype = base->_get_value(std::string("object->input_params->param")+number1+"->type", "");
            if(pname.empty())break;
            ocfg->_set_value(bname+"->"+pname, pdef, true);
        }
    }
    //Load current class
    for(int b=0;b<999999;++b)
    {
        snprintf(number, 7, "%d", b);
        std::string pname = object_class->_get_value(std::string("object->input_params->param")+number+"->name", "");
        std::string pdef = object_class->_get_value(std::string("object->input_params->param")+number+"->default", "");
        std::string ptype = object_class->_get_value(std::string("object->input_params->param")+number+"->type", "");
        if(pname.empty())break;
        ocfg->_set_value(std::string (class_name.toUtf8().data())+"->"+pname, pdef, true);
    }

    return true;
}
void MainWindow::on_AddObject_released()
{
    QString cname = ui->ObjectList->currentText();
    QString name = ui->objectName->text();
    if(!fill_object_config(name, cname))return;
    Scene *scn = get_scene("empty_scene");
    SceneObjectFabric fabrick;
    SceneObject *obj = fabrick[cname.toUtf8().data()];
    obj->set_video_name("Video");

    if(!obj->load(name.toUtf8().data(), "", "empty_scene", name.toUtf8().data(), filemanager))
    {
        std::cerr<<"Unable to load object: "<<obj->_get_error()<<std::endl;
    }
    if(!scn->add_object(name.toUtf8().data(), obj, 0))
    {
        std::cerr<<"Unable to add object"<<std::endl;
    }


    QTreeWidgetItem *item = new QTreeWidgetItem;
    item->setText(0, name);
    item->setText(1, cname);
    ui->AddedObjects->addTopLevelItem(item);
    ui->AddObject->setEnabled(false);
    objects.insert(name.toUtf8().data());
}

void MainWindow::on_AddedObjects_itemSelectionChanged()
{
    QTreeWidgetItem *item = ui->AddedObjects->selectedItems().at(0);
    ui->ObjectParamsWidget->show_object_params(std::string(item->text(0).toUtf8().data()), std::string(item->text(1).toUtf8().data()), &config_map);
}

void MainWindow::on_ObjectList_currentIndexChanged(QString val)
{
    std::string name=val.toUtf8().data();
    char number[9];
    for(int a=0;a<999999;++a)
    {
        snprintf(number, 9, "%d", a);
        if(objects.find(name+number)==objects.end())
        {
            ui->objectName->setText((name+number).c_str());
            ui->AddObject->setEnabled(true);
            return;
        }
    }

}

void MainWindow::on_objectName_textChanged(QString )
{
    std::string text=ui->objectName->text().toUtf8().data();
    if(objects.find(text)==objects.end() && !text.empty())ui->AddObject->setEnabled(true);
    else ui->AddObject->setEnabled(false);
}

void MainWindow::on_actionFonts_triggered()
{
    fonts->show();
}
