#include "objecttreeshower.h"
#include "ui_objecttreeshower.h"
#include <stdio.h>
#include <iostream>
#include "config.h"
#include "scene.h"
#include "scenemanager.h"
#include "sceneobjectfabric.h"
#include "sceneobject.h"

ObjectTreeShower::ObjectTreeShower(QWidget *parent) :
    QWidget(parent),
    m_ui(new Ui::ObjectTreeShower)
{
    m_ui->setupUi(this);
    m_ui->ObjectParams->setColumnWidth(0, 200);
    m_ui->ObjectParams->setColumnWidth(1, 40);
    m_ui->ObjectParams->setColumnWidth(2, 50);
    m_ui->ObjectParams->setColumnWidth(3, 50);
    m_ui->ObjectParams->setHeaderHidden(false);
}

ObjectTreeShower::~ObjectTreeShower()
{
    delete m_ui;
}

void ObjectTreeShower::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        m_ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void ObjectTreeShower::fill_one_object(Config_Interface *intf, QTreeWidgetItem *top)
{
    if(!intf)return;
    char number[7];
    QTreeWidgetItem *item = new QTreeWidgetItem;
    item->setText(0, intf->_get_value("object->name", "").c_str());

    for(int a=0;a<999999;++a)
    {
        snprintf(number, 7, "%d", a);
        std::string bname = intf->_get_value(std::string("object->base->base_class")+number+"->name", "");
        if(bname.empty())break;
        Config_Interface *intf1 = get_intf_by_name(bname);
        if(!intf1)continue;
        fill_one_object(intf1, item);
    }

    for(int a=0;a<999999;++a)
    {
        snprintf(number, 7, "%d", a);
        std::string pname = intf->_get_value(std::string("object->input_params->param")+number+"->name", "");
        std::string pdef = intf->_get_value(std::string("object->input_params->param")+number+"->default", "");
        std::string ptype = intf->_get_value(std::string("object->input_params->param")+number+"->type", "");
        if(pname.empty())break;
        QTreeWidgetItem *child = new QTreeWidgetItem;
        child->setText(0, pname.c_str());
        child->setText(1, ptype.c_str());
        child->setText(2, pdef.c_str());
        child->setText(3, pdef.c_str());
        item->addChild(child);
    }
    top->addChild(item);
}
Config_Interface* ObjectTreeShower::get_intf_by_name(std::string name)
{
    if(conf_map->find(name)==conf_map->end())return 0;
    std::string cname = conf_map->find(name)->second;
    return config_get_interface(cname);
}
void ObjectTreeShower::show_object_params(std::string oname, std::string otype, std::map<std::string, std::string> *conf_map_)
{
    conf_map=conf_map_;
    m_ui->ObjectParams->clear();
    QTreeWidgetItem *item = new QTreeWidgetItem;
    item->setText(0, oname.c_str());

    Config_Interface *intf = get_intf_by_name(otype);
    if(!intf)return;

    fill_one_object(intf, item);

    m_ui->ObjectParams->addTopLevelItem(item);

}

void ObjectTreeShower::on_ObjectParams_itemActivated(QTreeWidgetItem* item, int column)
{
    if(item->childCount()!=0)return;
    currently_edited_item = item;
    QString type = item->text(1);
    QString value = item->text(2);
    if(type == "text" || type == "int" || type == "float")
    {
        m_ui->TextValue->setEnabled(true);
        m_ui->BoolValue->setEnabled(false);
        m_ui->SetValue->setEnabled(true);
        m_ui->TextValue->setText(value);
    }
    else if(type == "bool")
    {
        m_ui->TextValue->setEnabled(false);
        m_ui->BoolValue->setEnabled(true);
        m_ui->SetValue->setEnabled(true);
        m_ui->BoolValue->setCurrentIndex((value=="true")?0:1);
    }
    else
    {
        m_ui->TextValue->setEnabled(false);
        m_ui->BoolValue->setEnabled(false);
        m_ui->SetValue->setEnabled(false);

    }
}

void ObjectTreeShower::on_SetValue_released()
{
    QString type = currently_edited_item->text(1);
    QString value;
    if(type=="bool")value=m_ui->BoolValue->currentIndex()?"true":"false";
    else value=m_ui->TextValue->text();
    currently_edited_item->setText(2, value);
    std::string value_name = currently_edited_item->text(0).toUtf8().data();
    std::string config_path = std::string(currently_edited_item->parent()->text(0).toUtf8().data())+"->"+value_name;

    QString name = m_ui->ObjectParams->topLevelItem(0)->text(0);
    Config_Interface *cfg=config_get_interface(name.toUtf8().data());
    cfg->_set_value(config_path, value.toUtf8().data(), true);

    std::cerr<<config_path<<std::endl;
    cfg->_save("/tmp/123.txt", filemanager);

    QString cname = m_ui->ObjectParams->topLevelItem(0)->child(0)->text(0);

    //Reload object
    Scene *scn = get_scene("empty_scene");
    SceneObjectFabric fabrick;
    scn->remove_object(name.toUtf8().data());

    SceneObject *obj = fabrick[cname.toUtf8().data()];
    obj->set_video_name("Video");
    if(!obj->load(name.toUtf8().data(), "", "empty_scene", name.toUtf8().data(), filemanager))
    {
        std::cerr<<"Unable to load object: "<<obj->_get_error()<<std::endl;
    }
    if(!scn->add_object(name.toUtf8().data(), obj, 0))
    {
        std::cerr<<"Unable to add object"<<std::endl;
    }
}
