/***************************************************************************
*   Copyright (C) 2005 by Denis A. Birukov aka Messenger of death         *
*   riper@fromru.com                                                      *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#ifndef VIDEO_H
#define VIDEO_H
#include <string>
#include <map>
#include "video_int.h"
#ifdef Revision
#undef Revision
#endif
#define Revision "video.h $Revision: 1.2 $"
/**
 * Управление устройствами видео вывода. Допустимо использование нескольких устройств.
 * @author Denis A. Birukov aka Messenger of death
 * @date 30.08.2005
 */
	/**
	 * Функция производит создание видео устройства и загрузку видео драйвера.
	 * @param name Имя создаваемого устройства
	 * @param driver Имя драйвера
	 * @param params Указатель на массив строк, содержащих параметры, которые должны быть переданы для инициализации драйвера.
	 * @param params_num количество передаваемых параметров.
	 * @param replace если установлен в true, то в случае, если устройство с заданным именем уже существует, оно будет удалено.
	 * @return true в случае успеха. false в случае неудачи. Причина неудачи может быть получена через _get_last_error() либо _get_error(); 
	 * @author Denis A. Birukov aka Messenger of death
	 * @date 30.08.2005
	 */
	extern "C" bool video_create ( std::string name, std::string driver, std::string *params, int params_num, bool replace = false );

	/**
	 * Функция удаляет видео устройство. При этом производится деинициализация драйвера.
	 * @param name Имя удаляемого устройства
	 * @return true в случае успеха. false в случае неудачи. Причина неудачи может быть получена через _get_last_error() либо _get_error()
	 * @author Denis A. Birukov aka Messenger of death
	 * @date 30.08.2005
	 */
	extern "C" bool video_remove ( std::string name );

	/**
	 * Возвращает указатель на интерфейс видео устройства
	 * @param name Имя видео устройства, интерфейс к которому необходимо получить
	 * @return Указатель на интерфейс видео устройства, 0 в случае ошибки. 
	 * @author Denis A. Birukov aka Messenger of death
	 * @date 30.08.2005
	 */
	extern "C" Video_Interface* video_get_interface ( std::string name );

	/**
	 * Функция возвращает ссылку на текст предыдущей ошибки.
	 * @return текст ошибки. Если ошибки небыло - строка будет пустой.
	 * @author Messenger of death
	 * @date 30.08.2005
	 */
	extern "C" std::string video_error();
	/**
	* Получение версии библиотеки
	* @return строку, содержащую версию библиотеки
	* @author Messenger of death
	* @date 30.08.2005
		 */
	extern "C" std::string video_version();

#endif
