//
// C++ Implementation: Ammo
//
// Description:
//
//
// Author: Messenger of death aka Denis A. Birukov <messengerofdeath@gmail.com>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "ammo.h"
#include "video.h"
#include "video_int.h"
#include "imageloader.h"
#include "signal_manager.h"
#include "scene.h"

Ammo::Ammo ( std::string name )
		: Image(),
		INIT_SLOT ( Ammo, this, _on_new_cycle )
{
	_object_name_ = name;
}


Ammo::~Ammo()
{
}


void Ammo::draw()
{
	Image::draw();
}

bool Ammo::init ( std::string video_name, std::string scene_name, double x_, double y_, const AmmoInfo &ammo )
{
	_video_name_ = video_name;
	_scene_name_ = scene_name;
	x = x_;
	y = y_;
	_damage_ = ammo._damage;
	_image_name_ = ammo._image_file;;

	Video_Interface *vid=video_get_interface ( _video_name_ );
	if ( !vid )
	{
		_set_error ( "Can't get video interface:"+video_error() );
		return false;
	}
	if ( !vid->_create_sprite ( _object_name_, _scene_name_, _image_name_, _scene_name_, true ) )
	{
		_set_error ( "Can't create sprite: "+vid->_get_error() );
		return false;
	}
	Video_Sprite *spr=vid->_get_sprite ( _object_name_, _scene_name_ );
	if ( !spr )
	{
		_set_error ( "Can't get sprite: "+vid->_get_error() );
		return false;
	}
	double xb = 0.5, yb = 0;
	spr->_set_base_point ( xb, yb );
	spr->_set_coord ( x, y );
	
	register_slot ( _scene_name_ + "->" + _object_name_ + "->" + "new_cycle", &_on_new_cycle );
	connect ( _scene_name_ + "->new_cycle", _scene_name_ + "->" + _object_name_ + "->" + "new_cycle" );
	_check_hit_.set_name ( _scene_name_ + "->check_hit" );
	_set_show(true);
	return true;
}

DEFINE_SLOT ( Ammo, _on_new_cycle )
{
	Video_Interface *vid=video_get_interface ( _video_name_ );
	if ( !vid )
	{
		_set_error ( "Can't get video interface:"+video_error() );
		return false;
	}
	Video_Sprite *spr=vid->_get_sprite ( _object_name_, _scene_name_ );
	if ( !spr )
	{
		_set_error ( "Can't get sprite: "+vid->_get_error() );
		return false;
	}
	y += ( double ) _timer_.get_elapsed_time() /0.5;

	spr->_set_coord ( x, y );
	if ( y >1 )
	{
		disconnect ( _scene_name_ + "->new_cycle", _scene_name_+ "->" + _object_name_ + "->new_cycle" );
		Scene *scn = get_scene ( _scene_name_ );
		if ( !scn ) abort();
		scn->remove_object ( _object_name_ );
		delete this;
		return false;
	}
	else
	{
		_check_hit_.clear_params();
		_check_hit_.add_params ( _object_name_ );
		_check_hit_.add_params ( ( float ) x );
		_check_hit_.add_params ( ( float ) y );
		_check_hit_.add_params ( ( int64_t ) _damage_ );
		if ( _check_hit_.rise() )
		{
			disconnect ( _scene_name_ + "->new_cycle", _scene_name_+ "->" + _object_name_ + "->new_cycle" );
			Scene *scn = get_scene ( _scene_name_ );
			if ( !scn ) abort();
			scn->remove_object ( _object_name_ );
			delete this;
			return false;
		}
	}
	return false;
}
DEFINE_SLOT( Ammo, _on_scene_reinit)
{
	disconnect ( _scene_name_ + "->new_cycle", _scene_name_+ "->" + _object_name_ + "->new_cycle" );
	unregister_slot ( _scene_name_ + "->" + _object_name_ + "->" + "new_cycle" );
	Scene *scn = get_scene ( _scene_name_ );
	if ( !scn ) abort();
	scn->remove_object ( _object_name_ );
	delete this;
	return false;
}

