/***************************************************************************
*   Copyright (C) 2005 by Messenger of death   *
*   Messengerofdeath@gmail.com   *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Library General Public License as       *
*   published by the Free Software Foundation; either version 2 of the    *
*   License, or (at your option) any later version.                       *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Library General Public     *
*   License along with this program; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "config.h"
#include "video.h"
#include "imageloader.h"
#include "textrenderer.h"
#include <stdio.h>
#include <string>
#include <iostream>
#include <ctime>
#include <fileio.h>

#include "defines.h"
#include "scenemanager.h"
#include "game_scene.h"
#include "sceneobjectfabric.h"
#include "herospaceship.h"
#include "enemyemitter.h"
#include "entername.h"
#include "scoresshower.h"
#include "menubackground.h"
#include "music.h"
#include "physfsfile.h"
#include "net_sync.h"

bool init_font(FileIO::FileManager *fmanager)
{
	Config_Interface *intf = config_get_interface ( "GameConfig" );
	if ( !intf )
	{
		std::cerr << "Error with config";
		return false;
	}
	std::string mainfont = intf->_get_value ( "Font->MainFont", "res/fonts/LiberationMono-Regular.ttf" );
	std::string lcdfont = intf->_get_value ( "Font->LCDFont", "res/fonts/lcd.ttf" );

	Text_Renderer* render_ttf=textrenderer_get ( "ttf" );
	if ( !render_ttf )
	{
		std::cerr<<"Error while getting ttf renderer"<<std::endl;
		return false;
	}
	if ( !render_ttf->init_font ( "MainFont", mainfont, fmanager ) )
	{
		std::cerr<<"Error while loading font:"<<render_ttf->_get_error() <<std::endl;
		return false;
	}

	if ( !render_ttf->init_font ( "LCDFont", lcdfont, fmanager ) )
	{
		std::cerr<<"Error while loading font:"<<render_ttf->_get_error() <<std::endl;
		return false;
	}
	return true;
}

bool init_loaders()
{
	if ( !imageloader_create ( "jpg", "libloader_jpg", true ) )
	{
		std::cerr<<"Failed to load JPEG loader \"libloader_jpg\""<<imageloader_error() <<std::endl;
		return false;
	}
	if ( !imageloader_create ( "png", "libloader_png", true ) )
	{
		std::cerr<<"Failed to load PNG loader \"libloader_png\""<<imageloader_error() <<std::endl;
		return false;
	}
	if ( !textrenderer_create ( "ttf", "libttf_renderer", true ) )
	{
		std::cerr<<"Failed to load TTF renderer \"librenderer_ttf\""<<textrenderer_error() <<std::endl;
		return false;
	}
	return true;
}
bool register_custom_scenes()
{
	SceneFabric::add_fabric ( "game_scene", &GameSceneFabrick );
	return true;
}
bool register_custom_objects()
{
	SceneObjectFabric::add_fabric ( "herospaceship", &HeroSpaceshipFabric );
	SceneObjectFabric::add_fabric ( "enemy_emitter", &EnemyEmitterFabric );
	SceneObjectFabric::add_fabric ( "enter_name", &EnterNameFabric );
	SceneObjectFabric::add_fabric ( "scores_shower", &ScoresShowerFabric );
	SceneObjectFabric::add_fabric ( "menu_background", &MenuBackgroundFabric );
	SceneObjectFabric::add_fabric ( "net_sync", &NetSyncFabric );
	return true;
}

bool init(FileIO::FileManager *fmanager)
{
	std::string mas[ 1 ];
	std::string params[ 1 ];

	if ( !config_create ( "GameConfig", "libtextconfig", true ) )
	{
		std::cerr << "Error while trying to create config from libtextconfig: " << config_error() << std::endl;
		return false;
	}
	Config_Interface *intf = config_get_interface ( "GameConfig" );
	if ( !intf )
	{
		std::cerr << "Error with config";
		return false;
	}
	intf->_load ( "cfg/Game.cfg", fmanager );

	//Load scores
	if ( !config_create ( "GameScores", "libtextconfig", true ) )
	{
		std::cerr << "Error while trying to create config from libtextconfig: " << config_error() << std::endl;
		return false;
	}
	intf = config_get_interface ( "GameScores" );
	if ( !intf )
	{
		std::cerr << "Error with config";
		return false;
	}
	intf->_load ( "res/data/scores.dat", fmanager );

	//Remote scores
	if ( !config_create ( "RemoteGameScores", "libtextconfig", true ) )
	{
		std::cerr << "Error while trying to create config from libtextconfig: " << config_error() << std::endl;
		return false;
	}
	intf = config_get_interface ( "RemoteGameScores" );
	if ( !intf )
	{
		std::cerr << "Error with config";
		return false;
	}
	intf->_load ( "res/data/remotescores.dat", fmanager );


	if ( !init_loaders() ) return false;
	if ( !init_font(fmanager) ) return false;

	if ( !config_create ( "VideoConfig", "libtextconfig", true ) )
	{
		std::cerr<< "Error while trying to create config from libtextconfig: " << config_error() << std::endl;
		return false;
	}
	intf = config_get_interface ( "VideoConfig" );
	if ( !intf )
	{
		std::cerr<< "Erro with config";
		return false;
	}
	intf->_load ( "cfg/Video.cfg", fmanager );
	std::string drv = intf->_get_value ( "Video->Driver", "libGL_mogul" );
	std::cerr<< "Video driver: " << drv << std::endl;

	std::string params1[ 3 ];
	params1[ 0 ] = "VideoConfig";
	params1[ 1 ] = "Video->";
	params1[ 2 ] = "ME_Video";

	if ( !video_create ( "Video", drv, params1, 3 ) )
	{
		std::cerr<< "Error while trying to create video from " << drv << ":" << video_error() << std::endl;
		return false;
	}
	Video_Interface *vid = video_get_interface ( "Video" );
	if ( !vid ) return 1;
	if ( !vid->_init() )
	{
		std::cerr<< "Error while trying to initialyze video: " << vid->_get_error() << std::endl;
		return false;
	}
	init ( "Video" );
	register_custom_scenes();
	register_custom_objects();
        init_music();
	srand(time(0));
	return true;
}

int main ( int argc, char *argv[] )
{
	FileIO::FileManager *filemanager;
	filemanager = new FileIO::FileManager(100);
	CreatePhysfsFile file(argv[0]);
	file.add_search_path( "res/data.zip" );
	FileIO::FileFabric f;
	f = file;
	filemanager->register_fileio( f );
	filemanager->register_fileio( &FileIO::CreatePosixFile );

	if ( !init(filemanager) ) return 1;
	load_scenes ( "res/scenes.scns", "Video", filemanager );

	Config_Interface *intf = config_get_interface ( "GameConfig" );
	std::string start_scene=intf->_get_value ( "Options->StartScene", "start_screen" );
	process_scene ( start_scene );

	Video_Interface *vid = video_get_interface ( "Video" );
	vid->_deinit();
	delete filemanager;
	return 0;
}

