//
// C++ Implementation: Enemy
//
// Description:
//
//
// Author: Messenger of death aka Denis A. Birukov <messengerofdeath@gmail.com>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "enemy.h"
#include "video.h"
#include "video_int.h"
#include "imageloader.h"
#include "signal_manager.h"
#include "scene.h"
#include "weaponbonus.h"
#include <math.h>
#include "eammo_info.h"
#include "enemyammo.h"
#include "me_signal.h"

int Enemy::_powerup_counter_ = 0;
int Enemy::_eammo_counter_ = 0;
Effect Enemy::_effect_ ;

Enemy::Enemy ( std::string name )
		: Image(),
		INIT_SLOT ( Enemy, this, _on_new_cycle ),
		INIT_SLOT ( Enemy, this, _on_check_hit )
{
	_object_name_ = name;
	_damage_done_ = 0;
	_dieing_state_ = false;
}


Enemy::~Enemy()
{}


bool Enemy::die()
{
	std::string surf_name = info._animation_set();
	if(surf_name != _last_surface_name_ && surf_name != "")
	{
//		std::cerr<<"Image:"<<surf_name<<std::endl;;
		//Выберем спрайт в зависимости от нанесенных повреждений кораблю и вызовем функцию рисования базового объекта
		Video_Interface *vid = video_get_interface ( _video_name_ );
		if ( !vid )
		{
			_set_error ( "Can't get video interface:" + video_error() );
			return true;
		}
		if ( !vid->_create_sprite ( _object_name_, _scene_name_, surf_name, _scene_name_ ) )
		{
			_set_error ( "Can't create sprite: " + vid->_get_error() );
			return true;
		}
		Video_Sprite *spr = vid->_get_sprite ( _object_name_, _scene_name_ );
		if ( !spr )
		{
			_set_error ( "Can't get sprite: " + vid->_get_error() );
			return true;
		}
		double xb = 0.5, yb = 0;
		spr->_set_base_point ( xb, yb );
		spr->_set_coord ( x, y );
		_last_surface_name_ = surf_name;
	}
	else if(surf_name == "") return false; //Надо убить объект финальной смертью
	return true;
}

void Enemy::alive()
{
	std::string surf_name = info._damage_set[_damage_done_];
	if(surf_name != _last_surface_name_)
	{
		//Выберем спрайт в зависимости от нанесенных повреждений кораблю и вызовем функцию рисования базового объекта
		Video_Interface *vid = video_get_interface ( _video_name_ );
		if ( !vid )
		{
			_set_error ( "Can't get video interface:" + video_error() );
			return;
		}
		if ( !vid->_create_sprite ( _object_name_, _scene_name_, surf_name, _scene_name_ ) )
		{
			_set_error ( "Can't create sprite: " + vid->_get_error() );
			return;
		}
		Video_Sprite *spr = vid->_get_sprite ( _object_name_, _scene_name_ );
		if ( !spr )
		{
			_set_error ( "Can't get sprite: " + vid->_get_error() );
			return;
		}
		double xb = 0.5, yb = 0;
		spr->_set_base_point ( xb, yb );
		spr->_set_coord ( x, y );
		_last_surface_name_ = surf_name;
	}
}

void Enemy::draw()
{
	if(_dieing_state_)
	{
		if(!die())//Убиваем финальной смертью
		{
			disconnect ( _scene_name_ + "->new_cycle", _scene_name_ + "->" + _object_name_ + "->new_cycle" );
			disconnect ( _scene_name_ + "->check_hit", _scene_name_ + "->" + _object_name_ + "->check_hit" );
			Scene *scn = get_scene ( _scene_name_ );
			if ( !scn ) abort();
			scn->remove_object ( _object_name_ );
			delete this;
			return;
		}
	}
	else alive();
	Image::draw();
}

bool Enemy::set_image ( std::string video_name, const EnemyInfo &enemy_info, std::string scene_name, double x_, double y_, FileIO::FileManager *fmanager )
{
	_video_name_ = video_name;
	_scene_name_ = scene_name;
	x = x_;
	y = y_;
	info = enemy_info;

	register_slot ( _scene_name_ + "->" + _object_name_ + "->" + "new_cycle", &_on_new_cycle );
	register_slot ( _scene_name_ + "->" + _object_name_ + "->" + "check_hit", &_on_check_hit );
	register_slot ( _scene_name_ + "->" + _object_name_ + "->" + "_on_scene_reinit", &_on_scene_reinit);

	connect ( _scene_name_ + "->new_cycle", _scene_name_ + "->" + _object_name_ + "->" + "new_cycle" );
	connect ( _scene_name_ + "->check_hit", _scene_name_ + "->" + _object_name_ + "->" + "check_hit" );
	connect ( _scene_name_ + "->reinit", _scene_name_ + "->" + _object_name_ + "->_on_scene_reinit");

	_add_score_.set_name ( _scene_name_ + "->add_score" );
	_add_score_.add_params ( ( int64_t ) info._hp ); //Amount of scores to add

	_check_enemy_hit_.set_name ( _scene_name_ + "->check_enemy_hit" );
	_fmanager_ = fmanager;
	_set_show(true);
	_chain_num = 0;
	_chain_timer_.reset();

	return true;
}

DEFINE_SLOT ( Enemy, _on_new_cycle )
{
	Video_Interface * vid = video_get_interface ( _video_name_ );
	if ( !vid )
	{
		_set_error ( "Can't get video interface:" + video_error() );
		return false;
	}
	Video_Sprite *spr = vid->_get_sprite ( _object_name_, _scene_name_ );
	if ( !spr )
	{
		_set_error ( "Can't get sprite: " + vid->_get_error() );
		return false;
	}
	double dt = _timer_.get_elapsed_time();
	y -= dt / info._speed;
	Video_Sprite *hero = vid->_get_sprite ( "hero", _scene_name_ );
	float hero_x = hero->_get_x();
	float hero_y = hero->_get_y();

	if ( info.move_style != 0 && hero && !_dieing_state_ )
	{
		double dx = dt / info._x_speed;
		if ( info.move_style == 1 )
		{
			if ( fabs ( x - hero_x ) > info.precision )
			{
				if ( x < hero_x ) x += dx;
				else if ( x > hero_x ) x -= dx;
			}
		}
		else
		{
			if ( fabs ( x - hero_x ) > info.precision )
			{
				if ( x < hero_x ) x -= dx;
				else if ( x > hero_x ) x += dx;
			}
		}
	}
	//Шмальнем если умеем
	if ( info.can_shoot && hero && !_dieing_state_)
	{
		
		double tm = info.ammo_info._timer_.get_elapsed_time_noreset();
		if ( tm > info.ammo_info._launch_delay )
		{
			if(info.can_shoot==2)
			{
				if(_chain_num > info._chain_length)
				{
					double t = _chain_timer_.get_elapsed_time_noreset();
					if(t>=info._chain_delay)
					{
						_chain_num = 0;
						_chain_timer_.reset();
					}
				}
			}
			
			if ( ! ( rand() % info._shoot_rate ) && _chain_num<=info._chain_length )
			{
				if(info.can_shoot==2)_chain_num++;
				char number[ 20 ];
				snprintf ( number, 20, "EAmmo%d", _eammo_counter_++ );
				EnemyAmmo *obj;
				obj = new EnemyAmmo ( number );
				obj->init ( _video_name_, _scene_name_, x, y, info.ammo_info, hero_x, hero_y );
				obj->_set_show(true);
				Scene *scn = get_scene ( _scene_name_ );
				if ( !scn ) return false;
				scn->add_object ( number, ( SceneObject* ) obj, 0 );
			}
			info.ammo_info._timer_.reset();
		}
	}
	spr->_set_coord ( x, y );
	if ( y < -1.5 )
	{
		disconnect ( _scene_name_ + "->new_cycle", _scene_name_ + "->" + _object_name_ + "->new_cycle" );
		disconnect ( _scene_name_ + "->check_hit", _scene_name_ + "->" + _object_name_ + "->check_hit" );
		Scene *scn = get_scene ( _scene_name_ );
		if ( !scn ) abort();
		scn->remove_object ( _object_name_ );
		delete this;
		return false;
	}
	else
	{
		if(_dieing_state_)return false;
		VideoRect rect;
		_get_active_area_rect ( &rect );
		_check_enemy_hit_.clear_params();
		_check_enemy_hit_.add_params ( _object_name_ );
		_check_enemy_hit_.add_params ( ( float ) rect.x_s );
		_check_enemy_hit_.add_params ( ( float ) rect.x_e );
		_check_enemy_hit_.add_params ( ( float ) rect.y_s );
		_check_enemy_hit_.add_params ( ( float ) rect.y_e );
		_check_enemy_hit_.add_params ( (int64_t) (info._hp - _damage_done_));
		if(_check_enemy_hit_.rise())
		{
			_effect_.play();
			_add_score_.rise();
			_dieing_state_ = true;

		}

	}
	return false;
}
DEFINE_SLOT ( Enemy, _on_check_hit )
{
	if(_dieing_state_)return false;
	try
	{
		std::string checker_name = args->string_args.at ( 0 );
		float checker_x = args->float_args.at ( 0 );
		float checker_y = args->float_args.at ( 1 );
		int damage_done = ( int ) args->int_args.at ( 0 );
		VideoRect rect;
		_get_active_area_rect ( &rect );
		if ( rect.x_s <= checker_x && checker_x <= rect.x_e && rect.y_s <= checker_y && checker_y <= rect.y_e )
		{
			_damage_done_ += damage_done;
			if ( info._hp > _damage_done_ ) return true;

			//Моб умер. Надо проверить и возможно создать оружейный поверап
			if ( ! ( rand() % info._drop_chance ) ) //Шанс 1 к числу в конфиге
			{
				Video_Interface * vid = video_get_interface ( _video_name_ );
				if ( !vid )
				{
					_set_error ( "Can't get video interface:" + video_error() );
					return false;
				}
				Video_Sprite *spr = vid->_get_sprite ( _object_name_, _scene_name_ );
				if ( !spr )
				{
					_set_error ( "Can't get sprite: " + vid->_get_error() );
					return false;
				}

				char number[ 100 ];
				snprintf ( number, 99, "Powerup%d", _powerup_counter_++ );
				WeaponBonus *obj;
				obj = new WeaponBonus ( number );
				obj->set_image ( _video_name_, "res/images/powerup.png", _scene_name_, x, y, _fmanager_ );
				Scene *scn = get_scene ( _scene_name_ );
				if ( !scn ) return false;
				scn->add_object ( number, ( SceneObject* ) obj, 0 );
			}
			_effect_.play();

			_add_score_.rise();
			_dieing_state_ = true;
			return true;
		}
		return false;
	}
	catch ( ... )
	{
		return false;
	}
}
DEFINE_SLOT ( Enemy, _on_scene_reinit )
{
	disconnect ( _scene_name_ + "->new_cycle", _scene_name_ + "->" + _object_name_ + "->new_cycle" );
	disconnect ( _scene_name_ + "->check_hit", _scene_name_ + "->" + _object_name_ + "->check_hit" );
	unregister_slot ( _scene_name_ + "->" + _object_name_ + "->" + "new_cycle" );
	unregister_slot ( _scene_name_ + "->" + _object_name_ + "->" + "check_hit" );
	Scene *scn = get_scene ( _scene_name_ );
	if ( !scn ) abort();
	scn->remove_object ( _object_name_ );
	delete this;
	return false;
}
