/***************************************************************************
 *   Copyright (C) 2007 by Denis A. Birukov   *
 *   Denis A. Birukov@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "enemyemitter.h"
#include "slot.h"
#include "me_signal.h"
#include "enemy.h"
#include "video_int.h"
#include "video.h"
#include "scene.h"
#include "imageloader.h"
#include <stdio.h>
#include <assert.h>

ObjectFabric<EnemyEmitter> EnemyEmitterFabric;

EnemyEmitter::EnemyEmitter()
		: SceneObject(),
		INIT_SLOT ( EnemyEmitter, this, on_new_cycle )
{
	_chance_count_ = 0;
	_enemy_counter_ = 0;
}


EnemyEmitter::~EnemyEmitter()
{
}


bool EnemyEmitter::load ( Scene *scene, std::string storage_name, std::string storage_prefix, std::string scene_name, std::string object_name, FileIO::FileManager *fmanager )
{
	bool ret = SceneObject::load ( scene, storage_name, storage_prefix, scene_name, object_name, fmanager );
	Config_Interface *intf = config_get_interface ( storage_name );
	_enemy_config_file_ = intf->_get_value ( storage_prefix+"enemy_file", "" );
	_chanse_config_file_ = intf->_get_value ( storage_prefix+"chance_file", "" );
	_eammo_config_file_ = intf->_get_value ( storage_prefix+"eammo_file", "" );
	_emit_rate_ = atof ( intf->_get_value ( storage_prefix+"emit_rate", "" ).c_str() );
	_max_emit_rate_ = atof ( intf->_get_value ( storage_prefix+"max_emit_rate", "" ).c_str() );
	_emit_change_speed_ = atof ( intf->_get_value ( storage_prefix+"emit_change_speed", "" ).c_str() );
	_explosion_fx_ = intf->_get_value ( storage_prefix + "explosion_fx", "" );
	Enemy::_effect_.load ( _explosion_fx_ );
	_base_emit_rate_ = _emit_rate_;

	_fmanager_ = fmanager;

//Load ammo config
	std::cerr<<"EnemyAmmoConf:"<<_eammo_config_file_<<std::endl;
	if ( !config_create ( _eammo_config_file_, "libtextconfig" ) )
	{
		std::cerr << "Error while trying to create config from libtextconfig: " << config_error() << std::endl;
		return false;
	}
	//Load config
	Config_Interface *EAI = config_get_interface ( _eammo_config_file_ );
	EAI->_load ( _eammo_config_file_, fmanager );

//Load each ammo description
	_eammo_count_ = atol ( EAI->_get_value ( "Ammo->AmmoCount", "0" ).c_str() );
	for ( int a=0; a < _eammo_count_; ++a )
	{
		char prefix[100]="";
		snprintf ( prefix, 99, "Ammo%d->", a );
		_ammos_.push_back ( EAmmoInfo() );
		if ( !_ammos_.back().load ( _eammo_config_file_, prefix, scene_name, _video_name_, fmanager ) )
		{
			std::cerr<<"Error while loading: "<<prefix<<" from "<<_eammo_config_file_<<std::endl;
			return false;
		}
	}

//Load enemy config
	std::cerr<<"EnemyConf:"<<_enemy_config_file_<<std::endl;
	if ( !config_create ( _enemy_config_file_, "libtextconfig", true ) )
	{
		std::cerr << "Error while trying to create config from libtextconfig: " << config_error() << std::endl;
		return false;
	}
	//Load config
	Config_Interface *ECI = config_get_interface ( _enemy_config_file_ );
	ECI->_load ( _enemy_config_file_, fmanager );

//Load each enemy description
	_enemy_count_ = atol ( ECI->_get_value ( "Enemyes->EnemyCount", "0" ).c_str() );
	for ( int a=0; a < _enemy_count_; ++a )
	{
		char prefix[100]="";
		snprintf ( prefix, 99, "Enemy%d->", a );
		//Получим ID патрона и передадим в информацию о вражине
		int ammo_id = atol ( ECI->_get_value ( std::string ( prefix ) + "ammo", "0" ).c_str() );
		_enemyes_.push_back ( EnemyInfo() );
		if ( !_enemyes_.back().load ( _enemy_config_file_, prefix, scene_name, _video_name_, _ammos_[ammo_id], fmanager ) )
		{
			std::cerr<<"Error while loading: "<<prefix<<" from "<<_enemy_config_file_<<std::endl;
			return false;
		}
	}

//Load chanse config
	std::cerr<<"ChanceConf:"<<_chanse_config_file_<<std::endl;
	if ( !config_create ( _chanse_config_file_, "libtextconfig", true ) )
	{
		std::cerr << "Error while trying to create config from libtextconfig: " << config_error() << std::endl;
		return false;
	}
	//Load config
	Config_Interface *CCI = config_get_interface ( _chanse_config_file_ );
	CCI->_load ( _chanse_config_file_, fmanager );
//Load each chance description
	_chance_count_ = atol ( ECI->_get_value ( "chance->count", "0" ).c_str() );
	for ( int a=0; a < _enemy_count_; ++a )
	{
		char prefix[100]="";
		snprintf ( prefix, 99, "chance%d->", a );
		EnemyChance en;
		if ( !en.load ( _chanse_config_file_, prefix, scene_name, _video_name_ ) )
		{
			std::cerr<<"Error while loading: "<<prefix<<" from "<<_chanse_config_file_<<std::endl;
			return false;
		}
		_chanses.insert ( std::make_pair ( en.get_start_time(), en ) );
	}


	register_slot ( _scene_name_ + "->" + _object_name_ + "->" +"on_new_cycle", &on_new_cycle );
	return ret;
}

void EnemyEmitter::draw()
{
}

DEFINE_SLOT ( EnemyEmitter, on_new_cycle )
{
	if ( _timer_.get_elapsed_time_noreset() <= _emit_rate_ ) return false;
	if ( _emit_rate_ - _emit_change_speed_ > _max_emit_rate_ ) _emit_rate_-=_emit_change_speed_;
	_timer_.reset();

	char number[20];
	snprintf ( number, 20, "Enemy%d", _enemy_counter_++ );
	Enemy *obj;
	obj = new Enemy ( number );
	float x = ( float ) ( rand() %1024 ) /512. - 1.0;

	float tfs = _time_from_start_.get_elapsed_time_noreset();
	std::map<float, EnemyChance>::iterator it;
	EnemyChance *cur;
	for ( it = _chanses.begin(); it != _chanses.end(); ++it )
	{
		if ( it->second.get_start_time() <=tfs && ( it->second.get_stop_time() >tfs ) || ( it->second.get_stop_time() <0 ) )
		{
			cur = &it->second;
			break;
		}
	}
	EnemyChanses eid = cur->get_random_enemy_id();
	int mid = eid.enemy_id;
	assert ( mid <=_enemy_count_ );
	if ( mid>=_enemy_count_ )
	{
		mid=_enemy_count_-1;
	}
	obj->set_image ( _video_name_, _enemyes_[mid], _scene_name_, x, y, _fmanager_ );
	obj->_set_show ( true );
	Scene *scn = get_scene ( _scene_name_ );
	if ( !scn ) return false;
	scn->add_object ( number, ( SceneObject* ) obj, 0 );

	return false;
}
DEFINE_SLOT ( EnemyEmitter, _on_scene_reinit )
{
	_timer_.reset();
	_time_from_start_.reset();
	_emit_rate_ = _base_emit_rate_;
	return false;
}
