//
// C++ Implementation: herospaceship
//
// Description:
//
//
// Author: Messenger of death aka Denis A. Birukov <messengerofdeath@gmail.com>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "herospaceship.h"
#include "signal_manager.h"
#include "video.h"
#include "video_int.h"
#include "ammo.h"
#include "scene.h"
#include "scores.h"
#include <stdio.h>
#include <assert.h>
#include <math.h>

ObjectFabric<HeroSpaceship> HeroSpaceshipFabric;

HeroSpaceship::HeroSpaceship() :
		Image(),
		INIT_SLOT ( HeroSpaceship, this, _on_mouse_move ),
		INIT_SLOT ( HeroSpaceship, this, _on_shoot ),
		INIT_SLOT ( HeroSpaceship, this, _on_check_enemy_hit ),
		INIT_SLOT ( HeroSpaceship, this, _on_add_score ),
		INIT_SLOT ( HeroSpaceship, this, _on_check_hero_pickup ),
		_ammo_counter_ ( 0 ),
		_current_score_ ( 0 )
{
	_hp_ = 100;
}


HeroSpaceship::~HeroSpaceship()
{
}


bool HeroSpaceship::load (Scene *scene,  std::string storage_name, std::string storage_prefix, std::string scene_name, std::string object_name, FileIO::FileManager *fmanager )
{
	bool ret = Image::load ( scene, storage_name, storage_prefix, scene_name, object_name, fmanager );
	Config_Interface *intf = config_get_interface ( storage_name );

	_ammo_config_file_ = intf->_get_value ( storage_prefix + "ammo_file", "" );
        _move_speed_ = atof( intf->_get_value ( storage_prefix + "move_speed", "2.0" ).c_str());

	//Load config for ammo
	if ( !config_create ( _ammo_config_file_, "libtextconfig" ) )
	{
		std::cerr << "Error while trying to create config from libtextconfig: " << config_error() << std::endl;
		return false;
	}
	Config_Interface *ACI = config_get_interface ( _ammo_config_file_ );
	ACI->_load ( _ammo_config_file_, fmanager );

	//Load each ammo description
	_ammo_count_ = atol ( ACI->_get_value ( "Ammo->AmmoCount", "0" ).c_str() );
	for ( uint32_t a = 0; a < _ammo_count_; ++a )
	{
		char prefix[100] = "";
		snprintf ( prefix, 99, "Ammo%d->", a );
		_ammos_.push_back ( AmmoInfo() );
		if ( !_ammos_.back().load ( _ammo_config_file_, prefix, scene_name, _video_name_, fmanager ) )
		{
			std::cerr << "Error while loading: " << prefix << " from " << _ammo_config_file_ << std::endl;
			return false;
		}
	}
	std::string animation_set = intf->_get_value(storage_prefix+"animation_file", "");
	//Load animation set
	std::cerr<<"AnimationSet:"<<animation_set<<std::endl;
	if ( !config_create ( animation_set, "libtextconfig", true ) )
	{
		std::cerr << "Error while trying to create config from libtextconfig: " << config_error() << std::endl;
		return false;
	}
	//Load config
	Config_Interface *ASCI = config_get_interface ( animation_set);
	ASCI->_load ( animation_set, fmanager );
	if(!_animation_set.load(animation_set, "", scene_name, _video_name_, fmanager))return false;
	_base_surface_ = _object_name_;
	
	_current_ammo_ = atol ( intf->_get_value ( storage_prefix + "default_ammo", "0" ).c_str() );
	_default_ammo_ = _current_ammo_;
	register_slot ( _scene_name_ + "->" + _object_name_ + "->" + "on_mouse_move", &_on_mouse_move );
	register_slot ( _scene_name_ + "->" + _object_name_ + "->" + "on_shoot", &_on_shoot );
	register_slot ( _scene_name_ + "->" + _object_name_ + "->" + "on_check_enemy_hit", &_on_check_enemy_hit );
	register_slot ( _scene_name_ + "->" + _object_name_ + "->" + "on_check_hero_pickup", &_on_check_hero_pickup );
	register_slot ( _scene_name_ + "->" + _object_name_ + "->" + "on_add_score", &_on_add_score );

	SignalName sn1 = _scene_name_+"->"+_object_name_+"->set_damage";
	set_damage.set_name(sn1);
	sn1 = _scene_name_+"->"+_object_name_+"->set_score";
	set_score.set_name(sn1);
	_dieing_state_ = false;
	_set_show(true);
	return ret;
}

bool HeroSpaceship::die()
{
	std::string surf_name = _animation_set();
	if(surf_name != _last_surface_name_ && surf_name != "")
	{
		Video_Interface *vid = video_get_interface ( _video_name_ );
		if ( !vid )
		{
			_set_error ( "Can't get video interface:" + video_error() );
			return true;
		}
		if ( !vid->_create_sprite ( _object_name_, _scene_name_, surf_name, _scene_name_ ) )
		{
			_set_error ( "Can't create sprite: " + vid->_get_error() );
			return true;
		}
		Video_Sprite *spr = vid->_get_sprite ( _object_name_, _scene_name_ );
		if ( !spr )
		{
			_set_error ( "Can't get sprite: " + vid->_get_error() );
			return true;
		}
		double xb = 0.5, yb = 0;
		spr->_set_base_point ( xb, yb );
		spr->_set_coord ( x, y );
		_last_surface_name_ = surf_name;
	}
	else if(surf_name == "") return false;
	return true;
}

void HeroSpaceship::draw()
{
	if(_dieing_state_)
	{
		if(!die())
		{
			show=false;
		}
		Image::draw();
	}
	else if( _hp_ - _damage_ <= 0)
	{
		
		_scene_->set_mouse_cursor_state(true);
		if(add_score ( _current_score_ ))
		{
			SceneObject *object = _scene_->get_object("scores_add_window");
			if(!object)return;
			object->_set_show(true);
		}
		else
		{
			SceneObject *object = _scene_->get_object("scores_show_local_window");
			if(!object)return;
			object->_set_show(true);
			object = _scene_->get_object("scores_show_remote_window");
			if(!object)return;
			object->_set_show(true);
			object = _scene_->get_object("scores_show_menu_window");
			if(!object)return;
			object->_set_show(true);
		}
		_dieing_state_ = true;
	}
	else Image::draw();
}

DEFINE_SLOT ( HeroSpaceship, _on_mouse_move )
{
	if(_hp_<_damage_)return false;

	Video_Interface *vid = video_get_interface ( _video_name_ );
	if ( !vid )
	{
		_set_error ( "Can't get video interface:" + video_error() );
		return false;
	}
	float dx, dy;
	try
	{
		dx = args->float_args.at ( 2 );
                dy = args->float_args.at ( 3 );
	}
	catch ( ... )
	{
		return false;
	}

	Video_Sprite *spr = vid->_get_sprite ( _object_name_, _scene_name_ );
	if ( !spr )
	{
		_set_error ( "Can't get sprite: " + vid->_get_error() );
		return false;
	}
        double dt = _timer_.get_elapsed_time();
        double dmax = dt / _move_speed_;

        if ( dmax < fabs( dx ) ) dx = dx/fabs(dx) * dmax;
        if ( dmax < fabs( dy ) ) dy = dy/fabs(dy) * dmax;
        if ( x + dx >= -1.0 && x + dx <= 1.0 ) x += dx;
        if ( y + dy >= -1.0 && y + dy <= -0.5 ) y += dy;
        spr->_set_coord ( x, y );
	return false;
}

DEFINE_SLOT ( HeroSpaceship, _on_shoot )
{
	if(_hp_<_damage_)return false;
	if ( _current_ammo_ >= _ammo_count_ ) return false;
	if ( _ammo_shoot_timer_.get_elapsed_time_noreset() <= _ammos_[_current_ammo_]._launch_delay ) return false;
	_ammo_shoot_timer_.reset();

	Video_Interface *vid = video_get_interface ( _video_name_ );
	if ( !vid )
	{
		_set_error ( "Can't get video interface:" + video_error() );
		return false;
	}
	Video_Sprite *spr = vid->_get_sprite ( _object_name_, _scene_name_ );
	if ( !spr )
	{
		_set_error ( "Can't get sprite: " + vid->_get_error() );
		return false;
	}

	char number[20];
	snprintf ( number, 20, "Ammo%d", _ammo_counter_++ );
	Ammo *obj;
	obj = new Ammo ( number );
	obj->init ( _video_name_, _scene_name_, x, y, _ammos_[_current_ammo_] );
	obj->_set_show(true);
	Scene *scn = get_scene ( _scene_name_ );
	if ( !scn ) return false;
	scn->add_object ( number, ( SceneObject* ) obj, 0 );

	return false;
}

DEFINE_SLOT ( HeroSpaceship, _on_check_enemy_hit )
{
	if(_hp_<_damage_)return false;
	try
	{
		std::string checker_name = args->string_args.at ( 0 );
		float checker_x_s = args->float_args.at ( 0 );
		float checker_x_e = args->float_args.at ( 1 );
		float checker_y_s = args->float_args.at ( 2 );
		float checker_y_e = args->float_args.at ( 3 );
		int damage = args->int_args.at( 0 );
		Video_Interface *vid = video_get_interface ( _video_name_ );
		if ( !vid )
		{
			_set_error ( "Can't get video interface:" + video_error() );
			return false;
		}
		EVENT_TYPE ev;
		vid->_get_mouse_state ( LBUTTON, &ev );
		if ( ev == MOUSE_PRESS ) this->slot__on_shoot ( args );
		VideoRect rect;
		_get_active_area_rect ( &rect );
		if ( ( rect.x_s + ( rect.x_e - rect.x_s ) >= checker_x_s &&
		        rect.x_s <= checker_x_s + ( checker_x_e - checker_x_s ) ) &&
		        ( rect.y_s + ( rect.y_e - rect.y_s ) >= checker_y_s &&
		          rect.y_s <= checker_y_s + ( checker_y_e - checker_y_s ) ) )
		{
			_damage_ += damage;
			set_damage.clear_params();
			set_damage.add_params((int64_t) (_hp_ - _damage_));
			set_damage.rise();
			return true;
		}
		return false;
	}
	catch ( ... )
	{
		return false;
	}
}
DEFINE_SLOT ( HeroSpaceship, _on_add_score )
{
	if(_hp_<_damage_)return false;

	int new_value = 0;
	try
	{
		new_value = args->int_args.at ( 0 );
	}
	catch ( ... )
	{
		new_value = 1;
	}
	_current_score_ += new_value;
	set_score.clear_params();
	set_score.add_params((int64_t) (_current_score_));
	set_score.rise();

	return false;
}

void HeroSpaceship::disconnect_signals(bool is_disconnect_on_load)
{
	Image::disconnect_signals(is_disconnect_on_load);
	unregister_slot ( _scene_name_ + "->" + _object_name_ + "->" + "on_mouse_move" );
	unregister_slot ( _scene_name_ + "->" + _object_name_ + "->" + "on_shoot" );
	unregister_slot ( _scene_name_ + "->" + _object_name_ + "->" + "on_check_enemy_hit" );
	unregister_slot ( _scene_name_ + "->" + _object_name_ + "->" + "on_check_hero_pickup" );
	unregister_slot ( _scene_name_ + "->" + _object_name_ + "->" + "on_add_score" );
}
DEFINE_SLOT ( HeroSpaceship, _on_check_hero_pickup )
{
	if(_hp_<_damage_)return false;

	try
	{
		std::string checker_name = args->string_args.at ( 0 );
		float checker_x_s = args->float_args.at ( 0 );
		float checker_x_e = args->float_args.at ( 1 );
		float checker_y_s = args->float_args.at ( 2 );
		float checker_y_e = args->float_args.at ( 3 );
		Video_Interface *vid = video_get_interface ( _video_name_ );
		if ( !vid )
		{
			_set_error ( "Can't get video interface:" + video_error() );
			return false;
		}
		EVENT_TYPE ev;
		vid->_get_mouse_state ( LBUTTON, &ev );
		if ( ev == MOUSE_PRESS ) this->slot__on_shoot ( args );
		VideoRect rect;
		_get_active_area_rect ( &rect );
		if ( ( rect.x_s + ( rect.x_e - rect.x_s ) >= checker_x_s &&
		        rect.x_s <= checker_x_s + ( checker_x_e - checker_x_s ) ) &&
		        ( rect.y_s + ( rect.y_e - rect.y_s ) >= checker_y_s &&
		          rect.y_s <= checker_y_s + ( checker_y_e - checker_y_s ) ) )
		{
			if ( _current_ammo_+1 < _ammos_.size() ) _current_ammo_++;
			_damage_=(_damage_-5>0)?_damage_-5:0;
			set_damage.clear_params();
			set_damage.add_params((int64_t) (_hp_ - _damage_));
			set_damage.rise();

			return true;
		}
		return false;
	}
	catch ( ... )
	{
		return false;
	}
}

DEFINE_SLOT ( HeroSpaceship, _on_scene_reinit )
{
	_current_ammo_ = _default_ammo_;
	_current_score_ = 0;
	_ammo_shoot_timer_.reset();
	_damage_= 0;
	_dieing_state_ = false;
	Video_Interface *vid = video_get_interface ( _video_name_ );
	if ( !vid )
	{
		_set_error ( "Can't get video interface:" + video_error() );
		return false;
	}
	if ( !vid->_create_sprite ( _object_name_, _scene_name_, _object_name_, _scene_name_ ) )
	{
		_set_error ( "Can't create sprite: " + vid->_get_error() );
		return false;
	}
	Video_Sprite *spr = vid->_get_sprite ( _object_name_, _scene_name_ );
	if ( !spr )
	{
		_set_error ( "Can't get sprite: " + vid->_get_error() );
		return false;
	}
	spr->_set_base_point ( x_base, y_base );
	_animation_set.reset();
	_set_show(true);
	set_damage.clear_params();
	set_damage.add_params((int64_t)(_hp_ - _damage_));
	set_damage.rise();
	set_score.clear_params();
	set_score.add_params((int64_t)_current_score_);
	set_score.rise();
	SceneObject *object = _scene_->get_object("scores_add_window");
	if(!object)return false;
	object->_set_show(false);
	object = _scene_->get_object("scores_show_local_window");
	if(!object)return false;
	object->_set_show(false);
	object = _scene_->get_object("scores_show_remote_window");
	if(!object)return false;
	object->_set_show(false);
	object = _scene_->get_object("scores_show_menu_window");
	if(!object)return false;
	object->_set_show(false);
	_scene_->set_mouse_cursor_state(false);
	return false;
}
