//
// C++ Implementation: menubackground
//
// Description:
//
//
// Author: Messenger of death aka Denis A. Birukov <messengerofdeath@gmail.com>, (C) 2009
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "menubackground.h"
#include "slot.h"
#include "me_signal.h"
#include "enemy.h"
#include "video_int.h"
#include "video.h"
#include "scene.h"
#include "imageloader.h"
#include "me_time.h"

ObjectFabric<MenuBackground> MenuBackgroundFabric;

MenuBackground::MenuBackground() :
		SceneObject(),
		INIT_SLOT( MenuBackground, this, _on_new_frame )
{}


MenuBackground::~MenuBackground()
{}

DEFINE_SLOT ( MenuBackground, _on_new_frame )
{
	if ( tm.get_elapsed_time_noreset() < 0.01 ) return false;
	tm.reset();
	Video_Interface *vid = video_get_interface ( _video_name_ );
	if ( !vid )
	{
		_set_error( "Can't get video interface:" + video_error() );
		return false;
	}
	Video_Particles *particles = vid->_get_particles( _object_name_, _scene_name_ );

	for ( int a = 0;a < _count_;++a )
	{
		if ( _data_[ a ].alfa_direction == 1 ) _data_[ a ].color.set_a( _data_[ a ].color.ai - 1 );
		if ( _data_[ a ].alfa_direction == 2 ) _data_[ a ].color.set_a( _data_[ a ].color.ai + 1 );
		if ( _data_[ a ].color.ai == 0 && _data_[ a ].alfa_direction == 1 ) _data_[ a ].alfa_direction = 2;
		if(_data_[a].color.ai > 254*0.9 && _data_[a].alfa_direction == 2)_data_[ a ].alfa_direction = 0;
		
		int rval = rand() % _count_;
		if(hidestar_delay.get_elapsed_time_noreset()> 0.05)
		{
			hidestar_delay.reset();
			if(!_data_[rval].alfa_direction)_data_[rval].alfa_direction = 1;
		}
		particles->_set_particle( a, _data_[ a ].x, _data_[ a ].y, _data_[ a ].color.ri, _data_[ a ].color.gi, _data_[ a ].color.bi, _data_[ a ].color.a );
	}
	return false;
}

bool MenuBackground::load( Scene *scene, std::string storage_name, std::string storage_prefix, std::string scene_name, std::string object_name, FileIO::FileManager *fmanager )
{
	bool ret = SceneObject::load ( scene, storage_name, storage_prefix, scene_name, object_name, fmanager );
	if ( !ret ) return ret;
	Config_Interface *intf = config_get_interface ( storage_name );
	_fname_ = intf->_get_value ( storage_prefix + "spark_image", "" );
	_count_ = atol( intf->_get_value ( storage_prefix + "count", "" ).c_str() );
	_size_ = atol( intf->_get_value ( storage_prefix + "size", "" ).c_str() );

	//Загрузим картинку
	Video_Interface *vid = video_get_interface ( _video_name_ );
	if ( !vid )
	{
		_set_error( "Can't get video interface:" + video_error() );
		return false;
	}
	if ( !imageloader_loadimage_noext ( _video_name_, _fname_, _scene_name_, _object_name_, fmanager, true ) )
	{
		_set_error( "Can't load image:" + imageloader_error() );
		return false;
	}

	//Создадим частицы
	if ( !vid->_create_particles ( _object_name_, _scene_name_, _object_name_, _scene_name_, _count_, true ) ) return false;

	_data_ = new MenuBackgroundData[ _count_ ];

	//Проинициализируем частицы
	Video_Particles *particles = vid->_get_particles( _object_name_, _scene_name_ );
	particles->_set_particles_base_coord( 0, 0 );
	particles->_set_particle_size( _size_ );
	for ( int a = 0;a < _count_;++a )
	{
		double x_s, y_s;
		x_s = ( double ) ( rand() ) / ( double ) ( RAND_MAX / 2. );
		y_s = ( double ) ( rand() ) / ( double ) ( RAND_MAX / 2. );
		x_s -= 1.0;
		y_s -= 1.0;
		_data_[ a ].x = x_s;
		_data_[ a ].y = y_s;
		_data_[ a ].color.set_color( (float)0.9, 0.9, 1.0, 0.9 );
		_data_[ a ].alfa_direction = 0;
		particles->_set_particle( a, x_s, y_s, 200, 200, 200, 0.9);
	}

	register_slot( _scene_name_ + "->" + _object_name_ + "->on_new_frame", &_on_new_frame );
//	connect( _scene_name_ + "->new_cycle", _scene_name_ + "->" + _object_name_ + "->on_new_frame" );
	return true;
}

void MenuBackground::draw( )
{
	Video_Interface * vid = video_get_interface ( _video_name_ );
	if ( !vid )
	{
		_set_error( "Can't get video interface:" + video_error() );
		return ;
	}
	vid->_add_particles( _object_name_, _scene_name_, true );
}
