#ifndef WIN32
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#else
#include <winsock2.h>
#endif
#include "net_sync.h"
#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <openssl/sha.h>
#include <fcntl.h>
#include "textedit.h"
#include <iostream>
#include "scene.h"
#include "scenemanager.h"
#include "herospaceship.h"
#include "config_classes.h"
#include "video_int.h"
#include "textrenderer.h"
#include "video.h"
#include <errno.h>

extern std::string last_name;
ObjectFabric<NetSync> NetSyncFabric;
extern void unicode2utf8(const std::vector<int> &unicode, std::string &utf8);

void NetSync::generate_text(const std::string& text, unsigned int color)
{
	VideoColor clr;
	clr.set_color(color);
	Video_Interface *vid=video_get_interface ( _video_name_ );
	if ( !vid )
	{
		_set_error(std::string(__FILE__)+": Can't get video interface:"+video_error());
		return;
	}

	Text_Renderer *renderer = textrenderer_get(renderer_type);
	if ( !renderer )
	{
		_set_error(std::string(__FILE__)+": Can't get renderer:"+textrenderer_error());
		return;
	}
	if( !renderer->render_text_line ( _video_name_, font, text, clr, font_size, _object_name_, _scene_name_, true ))
	{
		_set_error(std::string(__FILE__)+": Can't render text:"+renderer->_get_error());
		return;
	}
	if(!vid->_create_sprite(_object_name_, _scene_name_, _object_name_, _scene_name_, true))
	{
		_set_error(std::string(__FILE__)+": Can't create sprite: "+vid->_get_error());
		return;
	}

	Video_Sprite *spr=vid->_get_sprite ( _object_name_, _scene_name_);
	if(!spr)
	{
		_set_error(std::string(__FILE__)+": Can't get sprite: "+vid->_get_error());
		return;
	}
	spr->_set_base_point ( Text::x_base, Text::y_base );
	spr->_set_coord(x, y);
}

bool NetSync::_on_mouse_release_handler(float x, float y, float dx, float dy, int64_t button_state)
{
#ifdef WIN32
	WSADATA wsaData;
	int iResult;
	iResult = WSAStartup(MAKEWORD(2, 0), &wsaData);
	if (iResult != 0) 
	{
	    std::cerr<<"WSAStartup failed: "<<iResult<<std::endl;
            return false;
	}
#endif
	VideoColor clr;
	Scene *game_scene = get_scene("game_scene");
	if(!game_scene)
	{
		generate_text("Внутренняя ошибка", bad_color);
#ifdef WIN32
		WSACleanup();
#endif
		return false;
	}
	SceneObject *obj = game_scene->get_object("hero");
	int score;
	if(obj)
	{
		HeroSpaceship *scor = dynamic_cast<HeroSpaceship*>(obj);
		score = scor->get_current_score();
	}
	else 
	{
		generate_text("Внутренняя ошибка", bad_color);
#ifdef WIN32
		WSACleanup();
#endif
		return false;
	}
	Config_Interface *scores = config_get_interface ( "RemoteGameScores" );
	if ( !scores )
	{
		generate_text("Внутренняя ошибка", bad_color);
#ifdef WIN32
		WSACleanup();
#endif
		return false;
	}

	int sock;
	struct hostent *hp;
	struct sockaddr_in serv_addr;
	if ((hp=gethostbyname("nsws.ru"))==0)
	{
		generate_text("Ошибка работы с сетью", bad_color);
#ifdef WIN32
		WSACleanup();
#endif
		return false;
	}
	memset(&serv_addr, 0, sizeof(serv_addr));
	memcpy(&serv_addr.sin_addr, hp->h_addr, hp->h_length);
	serv_addr.sin_family = hp->h_addrtype;
	serv_addr.sin_port = htons(3000);
	sock = socket (PF_INET, SOCK_STREAM, 0);
	if (!sock)
	{
		generate_text("Ошибка работы с сетью", bad_color);
#ifdef WIN32
		WSACleanup();
#endif
		return false;
		
	}
	if (connect(sock, (struct sockaddr *)&serv_addr, sizeof(serv_addr))<0)
	{
		generate_text("Ошибка работы с сетью", bad_color);
#ifdef WIN32
		WSACleanup();
#endif
		return false;
		
	}

	int r1=0;
	int s;
	if ((s=recv(sock, (char*)&r1, 4, 0))<4)
	{
		generate_text("Ошибка работы с сетью", bad_color);
#ifdef WIN32
		WSACleanup();
#endif
		return false;
	}

	int file = open("res/data.zip", O_RDONLY);
	lseek(file, r1, SEEK_SET);
	char sha1[20];
	char buf[4096];
	read(file, buf, 4096);
	close(file);

	SHA1((unsigned char*)buf, 4096, (unsigned char*)sha1);

	send(sock, sha1, 20, 0);
	char status;

	recv(sock, (char*)&status, 1, 0);
	if (status=='0')
	{
		//Send local score
		int len=last_name.length();
		send(sock, (char*)&score, 4, 0);
		send(sock, (char*)&len, 4, 0);
		send(sock, last_name.c_str(), len, 0);
		

		char prefix[100];
		//Receive new scores
		for (int a=0;a<10;++a)
		{
			int score=0;
			int name_len = 0;
			char name[40];
			memset(name, 0, 40);
			recv(sock, (char*)&score, 4, 0);
			recv(sock, (char*)&name_len, 4, 0);
			recv(sock, name, name_len, 0);
			char cval[100];
			snprintf ( prefix, 99, "Score%d->Score", a );
			snprintf ( cval, 99, "%d", score );
			scores->_set_value ( prefix, cval, true );
			snprintf ( prefix, 99, "Score%d->Name", a );
			scores->_set_value ( prefix, name, true );
		}
		std::string fname ( "res/data/remotescores.dat" );
		scores->_save ( fname, _fmanager_);

		generate_text("Рекорды успешно отправлены", ok_color);
	}
	else 
	{
		generate_text("Ошибка отправки рекордов", bad_color);
	}
	
	close(sock);
#ifdef WIN32
	WSACleanup();
#endif
	
	return false;
}
DEFINE_SLOT ( NetSync, _on_scene_reinit )
{
	generate_text(text, color);
	return false;
}

