#include "physfsfile.h"
#include <physfs.h>
#define THROW(reason, code) throw(FileIO::FileIOError(reason, code))

using namespace FileIO;

int PhysfsFile::read ( void *buf, int size )
{
	return PHYSFS_read (_file_, buf, 1, size);
}
int PhysfsFile::write ( const void *buf, int size )
{
	THROW("Write is not supported by physfs", FileIO::WRITE_ERROR);
}
void PhysfsFile::seek ( int64_t position, FILE_POSITION whence )
{
	if(whence != FileIO::POS_SET)THROW("Only seek set supported at this time ", FileIO::SEEK_ERROR);
	PHYSFS_seek(_file_, position);
}
void* PhysfsFile::mmap()
{
	int fsize = get_file_size();
	_map_cache_ = new char[fsize+1];
	read(_map_cache_, fsize);
	_map_cache_[fsize] = 0;
	return _map_cache_;
}
void PhysfsFile::reset()
{
	PHYSFS_seek(_file_, 0);
}
PhysfsFile::~PhysfsFile()
{
	if(_map_cache_)delete[] _map_cache_;
}
uint64_t PhysfsFile::get_file_size() const
{
	return PHYSFS_fileLength(_file_);
}

PhysfsFile::PhysfsFile ( const std::string &filename, READ_MODE mode, PHYSFS_file *file ) : File(filename, mode), _filename_ ( filename ), _file_mode_ ( mode ), _file_size_ ( 0 ), _file_(file)
{
	_map_cache_ = 0;
}
CreatePhysfsFile::CreatePhysfsFile(char *argv0)
{
	PHYSFS_init(argv0);
}
File* CreatePhysfsFile::operator()( const std::string &filename, READ_MODE mode )
{
	if(mode!=FileIO::READ_ONLY)return 0;
	PHYSFS_file *file;
	file = PHYSFS_openRead(filename.c_str());
	if(!file) return 0;
	return new PhysfsFile(filename, mode, file);
}
void CreatePhysfsFile::add_search_path(const std::string &path, bool is_insert_first)
{
	PHYSFS_addToSearchPath(path.c_str(), is_insert_first?0:1);
}

