//
// C++ Implementation: WeaponBonus
//
// Description:
//
//
// Author: Messenger of death aka Denis A. Birukov <messengerofdeath@gmail.com>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "enemy.h"
#include "video.h"
#include "video_int.h"
#include "imageloader.h"
#include "signal_manager.h"
#include "scene.h"
#include "weaponbonus.h"

WeaponBonus::WeaponBonus ( const std::string &name )
		: Image(),
		INIT_SLOT ( WeaponBonus, this, _on_new_cycle )
{
	_object_name_ = name;
	_flash_timer_.reset();
}


WeaponBonus::~WeaponBonus()
{
}


void WeaponBonus::draw()
{
	Image::draw();
}

bool WeaponBonus::set_image ( const std::string &video_name, const std::string &image_file, const std::string &scene_name, double x_, double y_, FileIO::FileManager *fmanager )
{
	_video_name_ = video_name;
	_scene_name_ = scene_name;
	_image_name_ = image_file;
	x = x_;
	y = y_;
	Video_Interface *vid = video_get_interface ( _video_name_ );
	if ( !vid )
	{
		_set_error ( "Can't get video interface:" + video_error() );
		return false;
	}
	bool load_state;
	load_state=imageloader_loadimage ( "png", video_name, _image_name_, scene_name, _object_name_, fmanager, true );
	if ( !load_state )
	{
		return false;
	}

	if ( !vid->_create_sprite ( _object_name_, _scene_name_, _object_name_, _scene_name_, false ) )
	{
		_set_error ( "Can't create sprite: " + vid->_get_error() );
		return false;
	}
	Video_Sprite *spr = vid->_get_sprite ( _object_name_, _scene_name_ );
	if ( !spr )
	{
		_set_error ( "Can't get sprite: " + vid->_get_error() );
		return false;
	}
	double xb = 0.5, yb = 0.5;
	spr->_set_base_point ( xb, yb );
	spr->_set_coord ( x, y );

	register_slot ( _scene_name_ + "->" + _object_name_ + "->" + "new_cycle", &_on_new_cycle );
	connect ( _scene_name_ + "->new_cycle", _scene_name_ + "->" + _object_name_ + "->" + "new_cycle" );

	_check_hero_pickup_.set_name ( _scene_name_ + "->check_hero_pickup" );
	_flash_timer_.reset();
	_set_show(true);
	return true;
}

DEFINE_SLOT ( WeaponBonus, _on_new_cycle )
{
	Video_Interface *vid = video_get_interface ( _video_name_ );
	if ( !vid )
	{
		_set_error ( "Can't get video interface:" + video_error() );
		return false;
	}
	Video_Sprite *spr = vid->_get_sprite ( _object_name_, _scene_name_ );
	if ( !spr )
	{
		_set_error ( "Can't get sprite: " + vid->_get_error() );
		return false;
	}
	float fdt = _flash_timer_.get_elapsed_time_noreset();
	if(fdt>=0.25)_flash_timer_.reset();
	if(fdt<=0.125)spr->_set_alpha( 1.0-fdt*5);
	else spr->_set_alpha( fdt*5 );
	
	y -= _timer_.get_elapsed_time() / 1.5;

	spr->_set_coord ( x, y );
	if ( y < -1.1 )
	{
		disconnect ( _scene_name_ + "->new_cycle", _scene_name_ + "->" + _object_name_ + "->new_cycle" );
		Scene *scn = get_scene ( _scene_name_ );
		if ( !scn ) return false;
		scn->remove_object ( _object_name_ );
		delete this;
		return false;
	}
	else
	{
		VideoRect rect;
		_get_active_area_rect ( &rect );
		_check_hero_pickup_.clear_params();
		_check_hero_pickup_.add_params ( _object_name_ );
		_check_hero_pickup_.add_params ( ( float ) rect.x_s );
		_check_hero_pickup_.add_params ( ( float ) rect.x_e );
		_check_hero_pickup_.add_params ( ( float ) rect.y_s );
		_check_hero_pickup_.add_params ( ( float ) rect.y_e );
		if ( _check_hero_pickup_.rise() ) //Нас подняли и мы умрем
		{
			disconnect ( _scene_name_ + "->new_cycle", _scene_name_ + "->" + _object_name_ + "->new_cycle" );
			Scene *scn = get_scene ( _scene_name_ );
			if ( !scn ) return false;
			scn->remove_object ( _object_name_ );
			delete this;
			return false;

		}

	}
	return false;
}
